//============================================================================
// Name        : acat.cpp
// Author      : Martin Zak
//============================================================================

#include <iostream>
#include <string>
#include <set>
#include <cstdlib>
#include <unistd.h>
#include <exception>
#include <exception/AlibException.h>
#include <factory/AutomatonFactory.h>
#include <factory/RegExpFactory.h>
#include <factory/StringFactory.h>
#include <factory/ExceptionFactory.h>
#include <sax/SaxParseInterface.h>
#include <FromXMLParsers.h>
#include <sax/ParserException.h>


#define VERSION "0.0.1"
#define USAGE "Usage: catPDA [-c] [input]";

int main(int argc, char** argv) {
	if(argc == 2 && std::string(argv[1])=="-v" ) {
			std::cout << argv[0] << " version " << VERSION << "\n";
			std::cout << USAGE;
			std::cout << std::endl;
			return EXIT_SUCCESS;
		}

	int i=getopt(argc, argv, "t:");
	while(i!=-1) {
		switch(i){
		default:
			optind--;
			break;
		}
		i=getopt(argc, argv, "t:");
	}

	try {
		std::list<sax::Token> tokens;
		if(optind == argc) {
			std::string input(std::istreambuf_iterator<char>(std::cin), std::istreambuf_iterator<char>());
			sax::SaxParseInterface::parseMemory(input, tokens);
		} else {
			sax::SaxParseInterface::parseFile(argv[optind],tokens);
		}

		if (alib::FromXMLParsers::automatonParser.first(tokens)) {
			automaton::Automaton automaton = automaton::AutomatonFactory::fromTokens(tokens);
			automaton::AutomatonFactory::toStdout(automaton);
		} else if (alib::FromXMLParsers::regexpParser.first(tokens)) {
			regexp::RegExp regexp = regexp::RegExpFactory::fromTokens(tokens);
			regexp::RegExpFactory::toStdout(regexp);
		} else if (alib::FromXMLParsers::stringParser.first(tokens)) {
			string::String string = string::StringFactory::fromTokens(tokens);
			string::StringFactory::StringFactory::toStdout(string);
		} else if (alib::FromXMLParsers::exceptionParser.first(tokens)) {
			exception::AlibException exception = exception::ExceptionFactory::fromTokens(tokens);
			exception::ExceptionFactory::toStdout(exception);
		} else {
			throw exception::AlibException( "Expected root tag automaton, grammar, regexp, string or exception. Read: " + tokens.front().getData());
		}
	} catch(const exception::AlibException& exception) {
		exception::ExceptionFactory::toStdout(exception);
		return 1;
	} catch(...) {
		return 127;
	}
	return 0;
}
