/*
 * Brzozowski.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef BRZOZOWSKI_H_
#define BRZOZOWSKI_H_

#include <map>
#include <set>
#include <string>
#include <deque>

#include <automaton/State.h>
#include <AlibException.h>

#include "../interface/IConversionFSM.h"
#include "../shared/Hexavigesimal.h"
#include "../include/macros.h"

#include "RegExpDerivation.h"
#include "RegExpOptimize.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Brzozowski algorithm (derivations of regular expressions).
 * Source: Melichar 2.110
 */
class Brzozowski : public IConversionFSM
{
public:
    /**
     * @param re Source regular expression.
     */
    Brzozowski( const regexp::RegExp & re );

    ~Brzozowski( void );

    /**
     * Performs conversion.
     * @return FSM equivalent to original regular expression.
     */
    automaton::FSM convert( void );

private:
    /**
     * input regexp
     */
    const regexp::RegExp & m_re;
};

} /* namespace conversions */

#endif /* BRZOZOWSKI_H_ */
