/*
 * BrzozowskiDerivationRRG.cpp
 *
 *  Created on: 6. 3. 2014
 *      Author: tomas
 */

#include "BrzozowskiDerivationRRG.h"

using namespace alib;
using namespace alphabet;
using namespace grammar;
using namespace regexp;

namespace conversions
{

BrzozowskiDerivationRRG::BrzozowskiDerivationRRG( const RegExp & re ) : m_re( re )
{

}

BrzozowskiDerivationRRG::~BrzozowskiDerivationRRG( void )
{

}

RightRegularGrammar BrzozowskiDerivationRRG::convert( void )
{
    RegExpOptimize opt;

    // 1.
    RegExp V = opt.optimize( m_re );

    set<Symbol> alphabet = m_re.getAlphabet( );

    set<RegExp> N = { V };
    deque<set<RegExp>> Ni;

    Ni.push_back( set<RegExp>( ) );
    Ni.at( 0 ).insert( V );

    int i = 1;

    // 2.
    while( ! Ni.at( i - 1 ).empty( ) )
    {
        Ni.push_back( set<RegExp>( ) ); // initialize set Q_i

        for( const auto & regexp : Ni.at( i - 1 ) )
        {
            RegExpDerivation deriv( regexp );

            for( const auto & a : alphabet )
            {
                RegExp derived = deriv.derivation( a );
                derived = opt.optimize( derived );

                if( ! isInSet( derived, N ) ) // if this state has already been found, do not add
                    Ni.at( i ).insert( derived );
            }
        }

        N.insert( Ni.at( i ).begin( ), Ni.at( i ).end( ) );
        i += 1;
    }

    // 3.
    RightRegularGrammar grammar;
    map<RegExp, Symbol> nonterminalMap;
    int nonterminalId = 0;

    for( const auto & s : alphabet )
        grammar.addTerminalSymbol( s.getSymbol( ) );

    for( const auto & r : N )
    {
        Symbol nt = grammar.createUniqueNonTerminalSymbol( toBase26( ++ nonterminalId ) );
        nonterminalMap.insert( pair<RegExp, Symbol>( r, nt ) );
    }

    for( const auto & r : N )
    {
        RegExpDerivation deriv( r );

        for( const auto & a : alphabet )
        {
            RegExp derived = deriv.derivation( a );
            derived = opt.optimize( derived );

            list<Symbol> leftSide = { nonterminalMap.find( r )->second };
            list<Symbol> rightSide = { a, nonterminalMap.find( derived )->second };

            Rule r( leftSide, rightSide );
            grammar.addRule( r );

            if( derived.containsEmptyString( ) )
            {
                list<Symbol> rightSide = { a };
                Rule r( leftSide, rightSide );
                grammar.addRule( r );
            }
        }
    }

    grammar.setStartSymbol( nonterminalMap.find( V )->second );

    if( V.containsEmptyString( ) )
    {
        list<Symbol> leftSide = { nonterminalMap.find( V )->second };
        list<Symbol> rightSide;

        if( grammar.isNonTerminalOnRightSideOfAnyRule( grammar.getStartSymbol( ) ) )
        {
            Symbol newStart = grammar.createUniqueNonTerminalSymbol( grammar.getStartSymbol( ).getSymbol( ), false );

            list<Symbol> leftSideNewStart = { newStart };
            for( const auto & rule : grammar.getRules( ) )
                if( rule.getLeftSide( ).front( ) == grammar.getStartSymbol( ) )
                    grammar.addRule( Rule( leftSideNewStart, rule.getRightSide( ) ) );

            grammar.setStartSymbol( newStart );
            grammar.addRule( Rule( leftSideNewStart, rightSide ) );
        }
        else
        {
            grammar.addRule( Rule ( leftSide, rightSide ) );
        }
    }

    return grammar;
}

} /* namespace conversions */
