/*
 * TM.h
 *
 *  Created on: Apr 24, 2013
 *      Author: martin
 */

#ifndef TM_H_
#define TM_H_

#include "../Automaton.h"
#include "TransitionTM.h"

namespace automaton {

using namespace std;
using namespace alphabet;

/**
 * Turing machine
 */
class TM: public automaton::Automaton {
protected:
	set<Symbol> tapeAlphabet;
	set<TransitionTM> transitions;
	Symbol blankSymbol;

public:
	TM();

	/**
	 * Adds symbol to the input alphabet.
	 * @param symbol Symbol to add
	 * @throws AutomatonException when symbol is not present in tape alphabet,
	 * when it's blank symbol or when it is already present in input alphabet
	 */
	virtual void addInputSymbol(const Symbol& symbol);

	/**
	 * @copydoc Automaton::removeState(const State&)
	 */
	virtual void removeState(const State& state);

	/**
	 * Removes input symbol from the input alphabet.
	 * @param symbol Symbol to remove
	 * @throws AutomatonException when symbol is part of the transition
	 */
	virtual void removeInputSymbol(const Symbol& symbol);

	/**
	 * Adds symbol to the tape alphabet.
	 * @param symbol Symbol to add
	 * @throw AutomatonException when Symbol is already present in tape alphabet
	 */
	void addTapeSymbol(const Symbol& symbol);

	/**
	 * Removes symbol from the tape alphabet.
	 * @param symbol Symbol to remove
	 * @throw AutomatonException when Symbol is not present in the tape alphabet,
	 * when it is used in transition or when it is present in input alphabet
	 */
	void removeTapeSymbol(const Symbol& symbol);

	/**
	 * @return tape alphabet
	 */
	const set<Symbol>& getTapeAlphabet() const;

	/**
	 * Adds transition to the automaton.
	 * @param transition transition to add
	 * @throws AutomatonException when some part of the transition is not present
	 * in the TM (state, tape symbol) or when transition already exists
	 */
	void addTransition(const TransitionTM& transition);

	/**
	 * Removes the transition from the TM.
	 * @param transition transition to remove
	 * @throws AutomatonException when transition is not present in the TM
	 */
	void removeTransition(const TransitionTM& transition);

	/**
	 * @return TM transitions
	 */
	const set<TransitionTM>& getTransitions() const;

	/**
	 * Sets the blank symbol = symbol representing empty cell of the tape
	 * @param symbol Symbol to set
	 */
	void setBlankSymbol(const Symbol& symbol);

	/**
	 * @return symbol representing empty cell of the tape
	 */
	const Symbol& getBlankSymbol() const;

	/**
	 * @copydoc Automaton::toXML(ostream&) const
	 */
	virtual void toXML(ostream& out) const;
};

} /* namespace automaton */
#endif /* TM_H_ */
