/*
 * GrammarParser.cpp
 *
 *  Created on: Nov 3, 2013
 *      Author: martin
 */

#include "GrammarParser.h"

#include "Grammar.h"
#include "../sax/ParserException.h"

namespace grammar {

UnknownGrammar GrammarParser::parse(list<Token>& input) {
	UnknownGrammar grammar;
	popToken(input, Token::START_ELEMENT, "grammar");

	while (true) {
		if (isToken(input, Token::END_ELEMENT, "grammar")) {
			break;
		} else if (isToken(input, Token::START_ELEMENT, "nonTerminalSymbols")) {
			parseNonTerminalSymbols(input, grammar);
		} else if (isToken(input, Token::START_ELEMENT, "terminalSymbols")) {
			parseTerminalSymbols(input, grammar);
		} else if (isToken(input, Token::START_ELEMENT, "rules")) {
			parseRules(input, grammar);
		} else if (isToken(input, Token::START_ELEMENT, "startSymbol")) {
			parseStartSymbol(input, grammar);
		} else {
			throw ParserException(Token("grammar", Token::END_ELEMENT), input.front());
		}
	}

	popToken(input, Token::END_ELEMENT, "grammar");

	return grammar;
}

void GrammarParser::parseNonTerminalSymbols(list<Token>& input, Grammar& grammar) {
	popToken(input, Token::START_ELEMENT, "nonTerminalSymbols");

	while (isToken(input, Token::START_ELEMENT, "symbol")) {
		grammar.addNonTerminalSymbol(parseSymbol(input));
	}

	popToken(input, Token::END_ELEMENT, "nonTerminalSymbols");
}

void GrammarParser::parseTerminalSymbols(list<Token>& input, Grammar& grammar) {
	popToken(input, Token::START_ELEMENT, "terminalSymbols");

	while (isToken(input, Token::START_ELEMENT, "symbol")) {
		grammar.addTerminalSymbol(parseSymbol(input));
	}

	popToken(input, Token::END_ELEMENT, "terminalSymbols");
}

void GrammarParser::parseRules(list<Token>& input, Grammar& grammar) {
	popToken(input, Token::START_ELEMENT, "rules");
	while (isToken(input, Token::START_ELEMENT, "rule")) {
		grammar.addRule(parseRule(input));
	}
	popToken(input, Token::END_ELEMENT, "rules");
}

Rule GrammarParser::parseRule(list<Token>& input) {
	popToken(input, Token::START_ELEMENT, "rule");

	Rule rule;

	rule.setLeftSide(parseSymbolList(input, "leftSide"));
	rule.setRightSide(parseSymbolList(input, "rightSide"));

	popToken(input, Token::END_ELEMENT, "rule");
	return rule;
}

void GrammarParser::parseStartSymbol(list<Token>& input, Grammar& grammar) {
	grammar.setStartSymbol(parseSymbol(input, "startSymbol"));
}

Symbol GrammarParser::parseSymbol(list<Token>& input, string tagName) {
	popToken(input, Token::START_ELEMENT, tagName);

	if (input.front().getType() == Token::CHARACTER) {
		Symbol symbol(input.front().getData());
		input.pop_front();
		popToken(input, Token::END_ELEMENT, tagName);
		return symbol;
	} else {
		throw ParserException(Token("", Token::CHARACTER), input.front());
	}
}

list<Symbol> GrammarParser::parseSymbolList(list<Token>& input, string tagName) {
	popToken(input,Token::START_ELEMENT, tagName);

	list<Symbol> symbols;
	while(isToken(input, Token::START_ELEMENT, "symbol")) {
		symbols.push_back(parseSymbol(input));
	}

	popToken(input,Token::END_ELEMENT, tagName);
	return symbols;
}

bool GrammarParser::isToken(list<Token>& input, Token::TokenType type, string data) {
	return input.front().getType() == type && input.front().getData() == data;
}

void GrammarParser::popToken(list<Token>& input, Token::TokenType type, string data) {
	if (isToken(input, type, data)) {
		input.pop_front();
	} else {
		throw ParserException(Token(data, type), input.front());
	}
}

} /* namespace grammar */
