/*
 * Alternation.h
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#ifndef ALTERNATION_H_
#define ALTERNATION_H_

#include <list>
#include "RegExpElement.h"

namespace regexp {

using namespace std;

/**
 * Represents alternation operator in the regular expression. Contains list of RegExpElement
 * as operands of the operator.
 */
class Alternation: public RegExpElement {
private:
	list<RegExpElement*> elements;
public:
	Alternation();
	Alternation(const Alternation& other);
	Alternation& operator =(const Alternation& other);
	~Alternation();

	/**
	 * @return list of operands
	 */
	list<RegExpElement*>& getElements();

	/**
	 * @return list of operands
	 */
	const list<RegExpElement*>& getElements() const;

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	RegExpElement* clone() const;

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator<(const Alternation&) const;
	virtual bool operator==(const Alternation&) const;

	/**
	 * @copydoc RegExpElement::getAlphabet()
	 */
	virtual void getAlphabet( std::set<alphabet::Symbol> & alphabet ) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */
#endif /* ALTERNATION_H_ */
