/*
 * RegExpPrinter.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: martin
 */

#include "RegExpPrinter.h"

namespace regexp {

const string RegExpPrinter::INDENTATION = "\t";

void RegExpPrinter::toXML(const RegExp& regexp, ostream& out) {
	out << "<regexp>\n";
	printElement(regexp.getRegExp(), out, INDENTATION);
	out << "</regexp>\n";
}

void RegExpPrinter::printElement(const RegExpElement* element, ostream& out, const string & prefix) {
	const Alternation* alternation = dynamic_cast<const Alternation*>(element);
	if (alternation) {
		printAlternation(alternation, out, prefix);
		return;
	}

	const Concatenation* concatenation = dynamic_cast<const Concatenation*>(element);
	if(concatenation) {
		printConcatenation(concatenation,out, prefix);
		return;
	}

	const Iteration* iteration = dynamic_cast<const Iteration*>(element);
	if (iteration) {
		printIteration(iteration, out, prefix);
		return;
	}

	const RegExpEpsilon* epsilon = dynamic_cast<const RegExpEpsilon*>(element);
	if (epsilon) {
		printEpsilon(epsilon, out, prefix);
		return;
	}

	const RegExpEmpty* empty = dynamic_cast<const RegExpEmpty*>(element);
	if (empty) {
		printEmpty(empty, out, prefix);
		return;
	}

	const RegExpSymbol* symbol = dynamic_cast<const RegExpSymbol*>(element);
	if (symbol) {
		printSymbol(symbol, out, prefix);
		return;
	}

}

void RegExpPrinter::printContent(const list<RegExpElement*>& content, ostream& out, const string & prefix) {
	for (auto element : content) {
		printElement(element, out, prefix);
	}
}

void RegExpPrinter::printAlternation(const Alternation* alternation, ostream& out, const string & prefix) {
	out << prefix << "<alternation>\n";
	printContent(alternation->getElements(), out, prefix + INDENTATION);
	out << prefix << "</alternation>\n";
}

void RegExpPrinter::printConcatenation(const Concatenation* concatenation, ostream& out, const string & prefix) {
	out << prefix <<"<concatenation>\n";
	printContent(concatenation->getElements(), out, prefix + INDENTATION);
	out << prefix <<"</concatenation>\n";

}

void RegExpPrinter::printIteration(const Iteration* iteration, ostream& out, const string & prefix) {
	out << prefix << "<iteration>\n";
	printElement(iteration->getElement(), out, prefix + INDENTATION);
	out << prefix << "</iteration>\n";
}

void RegExpPrinter::printSymbol(const RegExpSymbol* symbol, ostream& out, const string & prefix) {
	out << prefix << "<symbol>";
	out << symbol->getSymbol();
	out << "</symbol>\n";
}

void RegExpPrinter::printEpsilon(const RegExpEpsilon* symbol, ostream& out, const string & prefix) {
	out << prefix << "<epsilon>";
	out << "</epsilon>\n";
}

void RegExpPrinter::printEmpty(const RegExpEmpty* symbol, ostream& out, const string & prefix) {
	out << prefix << "<empty>";
	out << "</empty>\n";
}

} /* namespace regexp */

