/*
 * SaxInterface.cpp
 *
 *  Created on: 8.8.2012
 *      Author: martin
 */

#include "SaxInterface.h"

#include <string>
#include <cstring>
#include <cstdlib>
#include "../AlibException.h"

namespace sax {

using namespace std;
using namespace alib;

void SaxInterface::initSAXHandler(xmlSAXHandler& handler) {
	memset(&handler, 0, sizeof(handler));
	handler.initialized = XML_SAX2_MAGIC;

	handler.startElement = &sax::SaxInterface::startElement;
	handler.characters = &sax::SaxInterface::characters;
	handler.endElement = &sax::SaxInterface::endElement;
}

void SaxInterface::parseMemory(string xmlIn, list<Token>& out) {
	xmlSAXHandler handler;
	initSAXHandler(handler);

	int result = xmlSAXUserParseMemory(&handler, (void*) &out, xmlIn.c_str(), xmlIn.length());
	xmlCleanupParser();

	if (result != 0) {
		throw AlibException("Cannot parse the XML string.");
	}
}

void SaxInterface::parseFile(string filename, list<Token>& out) {
	xmlSAXHandler handler;
	initSAXHandler(handler);

	int result = xmlSAXUserParseFile(&handler, (void*) &out, filename.c_str());
	xmlCleanupParser();

	if (result != 0) {
		throw AlibException("Cannot parse the XML file " + filename);
	}

}

void SaxInterface::characters(void * userData, const xmlChar * ch, int len) {
	list<Token> &out = *((list<Token>*) userData);
	string tmp((char*) ch, len);

	for (unsigned int i = 0; i < tmp.length(); i++) {
		if (!isspace(tmp[i])) {
			Token token(tmp, Token::CHARACTER);
			out.push_back(token);
			break;
		}
	}
}

void SaxInterface::startElement(void* userData, const xmlChar* name, const xmlChar** attrs) {
	(void)attrs;

	list<Token> &out = *((list<Token>*) userData);
	out.push_back(Token((char*) name, Token::START_ELEMENT));
}

void SaxInterface::endElement(void * userData, const xmlChar * name) {
	list<Token> &out = *((list<Token>*) userData);
	out.push_back(Token((char*) name, Token::END_ELEMENT));
}
} /* namespace sax */
