//#include "stdafx.h"
#include <cppunit/CompilerOutputter.h>
#include <cppunit/extensions/TestFactoryRegistry.h>
#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/TestResultCollector.h>
#include <cppunit/TestResult.h>

#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/portability/Stream.h>
#include <cppunit/TestListener.h>
#include <cppunit/SourceLine.h>
#include <cppunit/Exception.h>

CPPUNIT_NS_BEGIN

class CPPUNIT_API TestProgressListener : public TestListener
{
public:
  TestProgressListener();

  virtual ~TestProgressListener();

  void startTest( Test *test );

  void addFailure( const TestFailure &failure );

  void endTest( Test *test );

  int getResult() const;

  void printResults() const;

private:
  TestProgressListener( const TestProgressListener &copy );

  void operator =( const TestProgressListener &copy );

private:
  int m_Failures;
  int m_Tests;
  int m_Assertions;
  bool m_lastTestFailed;
};

TestProgressListener::TestProgressListener() : m_Failures( 0 ), m_Tests(0), m_Assertions(0), m_lastTestFailed( false )
{
}

TestProgressListener::~TestProgressListener()
{
}

void TestProgressListener::startTest( Test * test )
{
	stdCOut() << test->getName() << ":" << "\n";
	stdCOut().flush();

	m_lastTestFailed = false;
	m_Tests++;
}

void TestProgressListener::addFailure( const TestFailure &failure )
{
	stdCOut() << (failure.isError() ? "error" : "assertion") << " : " << failure.failedTestName() << " : " << failure.sourceLine().lineNumber() << "\n";
	stdCOut() << "Exception " << failure.thrownException()->message().details();

	m_lastTestFailed = true;
	if(failure.isError()) m_Failures++; else m_Assertions++;
}

void TestProgressListener::endTest( Test * test)
{
	stdCOut() << "Result (" << test->getName() << ")";
	stdCOut().flush();

	if ( !m_lastTestFailed )
		stdCOut() <<  " : OK";
	else
		stdCOut() << " : Fail";
	stdCOut() << "\n\n";
}

int TestProgressListener::getResult() const {
	return m_Failures + m_Assertions;
}

void TestProgressListener::printResults() const {
	stdCOut() << "Overal result: Tests: " << m_Tests << " Assertions: " << m_Assertions << " Failures: " << m_Failures << "\n";
}

CPPUNIT_NS_END

int main(int , char*[])
{
  CppUnit::TestResult controller;

  CppUnit::TestResultCollector result;
  controller.addListener( &result );

  CppUnit::TestProgressListener progressListener;
  controller.addListener( &progressListener );

  // Get the top level suite from the registry
  CppUnit::Test *suite = CppUnit::TestFactoryRegistry::getRegistry().makeTest();

  // Adds the test to the list of test to run
  CppUnit::TextUi::TestRunner runner;
  runner.addTest( suite );

  // Change the default outputter to a compiler error format outputter
  runner.setOutputter( new CppUnit::CompilerOutputter( &runner.result(),
                                                       std::cerr ) );
  // Run the tests.
  runner.run( controller );

  progressListener.printResults();

  return progressListener.getResult();
}
