/*
 * Symbol.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_H_
#define SYMBOL_H_

#include "../std/visitor.hpp"
#include "SymbolBase.h"
#include <set>

namespace alphabet {

/**
 * Wrapper around automata.
 */
class Symbol { //TODO toto udělat v std templatované
protected:
	SymbolBase* symbol;
public:
	explicit Symbol(const SymbolBase& symbol);
	explicit Symbol(SymbolBase&& symbol);
	Symbol(const Symbol& other);
	Symbol(Symbol&&) noexcept;
	Symbol& operator=(const Symbol& other);
	Symbol& operator=(Symbol&& other) noexcept;
	virtual ~Symbol() noexcept;

	const SymbolBase& getSymbol() const;
	SymbolBase& getSymbol();

	void setSymbol(const SymbolBase& symbol);
	void setSymbol(SymbolBase&& symbol);
	
	bool operator<(const Symbol& other) const;

	bool operator!=(const Symbol& other) const;

	bool operator==(const Symbol& other) const;

	friend std::ostream& operator<<(std::ostream& os, const Symbol& symbol);

	operator std::string () const;

	/**
	 * Creates and adds unique state to grammar. If given state name is
	 * already used, appends apostrophe or integer suffix
	 * @param name name of the state
	 * @throws AutomatonException if symbol could not be created
	 * @return created symbol
	 */
	static alphabet::Symbol createUniqueSymbol(const alphabet::Symbol& base, const std::set<alphabet::Symbol>& Terminals, const std::set<alphabet::Symbol>& nonterminals);
};

} /* namespace alphabet */

#endif /* SYMBOL_H_ */

