/*
 * SymbolBase.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_BASE_H_
#define SYMBOL_BASE_H_

#include "../label/LabelBase.h"
#include <ostream>
	
namespace alphabet {

class LabeledSymbol;
class BlankSymbol;
class BottomOfTheStackSymbol;
class EndSymbol;

/**
 * Represents symbol in an alphabet.
 */
class SymbolBase : public std::elementBase<LabeledSymbol, BlankSymbol, BottomOfTheStackSymbol, EndSymbol> {
public:
	virtual ~SymbolBase() noexcept;

	virtual SymbolBase* clone() const = 0;
	virtual SymbolBase* plunder() && = 0; 

	virtual bool operator <(const SymbolBase& other) const = 0;
	virtual bool operator ==(const SymbolBase& other) const = 0;
	virtual bool operator >(const SymbolBase& other) const = 0;
	bool operator !=(const SymbolBase& other) const;

	virtual bool operator==(const LabeledSymbol& other) const;
	virtual bool operator==(const BlankSymbol& other) const;
	virtual bool operator==(const BottomOfTheStackSymbol& other) const;
	virtual bool operator==(const EndSymbol& other) const;

	virtual bool operator<(const LabeledSymbol& other) const;
	virtual bool operator<(const BlankSymbol& other) const;
	virtual bool operator<(const BottomOfTheStackSymbol& other) const;
	virtual bool operator<(const EndSymbol& other) const;

	friend std::ostream& operator<<(std::ostream&, const SymbolBase&);

	virtual void operator>>(std::ostream& out) const = 0;

	virtual operator std::string () const = 0;
};

} /* namespace alphabet */

#endif /* SYMBOL_BASE_H_ */
