#include "SymbolFromStringParser.h"
#include "../exception/AlibException.h"
#include "BlankSymbol.h"
#include "BottomOfTheStackSymbol.h"
#include "EndSymbol.h"
#include "LabeledSymbol.h"

namespace alphabet {

SymbolFromStringParser::SymbolFromStringParser(std::stringstream& input) : m_SymbolLexer(input), m_LabelParser(input) {

}

Symbol SymbolFromStringParser::parse() {
	return parse(std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END}));
}

Symbol SymbolFromStringParser::parse(const std::set<FEATURES>& features) {
	SymbolFromStringLexer::Token token = m_SymbolLexer.token();
	switch(token.type) {
	case SymbolFromStringLexer::TokenType::BLANK:
		if(!features.count(FEATURES::BLANK)) throw exception::AlibException();
		return Symbol(BlankSymbol());
	case SymbolFromStringLexer::TokenType::BOTTOM:
		if(!features.count(FEATURES::BOTTOM)) throw exception::AlibException();
		return Symbol(BottomOfTheStackSymbol());
	case SymbolFromStringLexer::TokenType::END:
		if(!features.count(FEATURES::END)) throw exception::AlibException();
		return Symbol(EndSymbol());
	case SymbolFromStringLexer::TokenType::ERROR:
		if(!features.count(FEATURES::LABELED)) throw exception::AlibException();
		return Symbol(LabeledSymbol(m_LabelParser.parse()));
	case SymbolFromStringLexer::TokenType::TEOF:
		throw exception::AlibException();
	}
	throw exception::AlibException();
}

bool SymbolFromStringParser::next() {
	SymbolFromStringLexer::Token token = m_SymbolLexer.next().token();
	if(token.type == SymbolFromStringLexer::TokenType::BLANK || token.type == SymbolFromStringLexer::TokenType::BOTTOM || token.type == SymbolFromStringLexer::TokenType::END) {
		return true;
	} else {
		return false;
	}
}

bool SymbolFromStringParser::first() {
	SymbolFromStringLexer::Token token = m_SymbolLexer.next().token();
	if(token.type == SymbolFromStringLexer::TokenType::BLANK || token.type == SymbolFromStringLexer::TokenType::BOTTOM || token.type == SymbolFromStringLexer::TokenType::END) {
		return true;
	} else {
		return token.type == SymbolFromStringLexer::TokenType::ERROR && m_LabelParser.first();
	}
}

bool SymbolFromStringParser::last() {
	SymbolFromStringLexer::Token token = m_SymbolLexer.next().token();
	if(token.type == SymbolFromStringLexer::TokenType::TEOF) {
		return true;
	} else {
		return false;
	}
}

} /* namespace alphabet */
