/*
 * SymbolFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "SymbolFromXMLParser.h"

#include "../sax/ParserException.h"
#include "BlankSymbol.h"
#include "BottomOfTheStackSymbol.h"
#include "EndSymbol.h"
#include "LabeledSymbol.h"
#include "../label/Label.h"

#include "../FromXMLParsers.h"

namespace alphabet {

Symbol SymbolFromXMLParser::parse(std::list<sax::Token>& input) const {
	return parse(input, std::set<FEATURES>({FEATURES::LABELED, FEATURES::BLANK, FEATURES::BOTTOM, FEATURES::END}));
}

Symbol SymbolFromXMLParser::parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol")) {
		if(!features.count(FEATURES::LABELED)) throw exception::AlibException();
		return parseLabeledSymbol(input);
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol")) {
		if(!features.count(FEATURES::BLANK)) throw exception::AlibException();
		return parseBlankSymbol(input);
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol")) {
		if(!features.count(FEATURES::BOTTOM)) throw exception::AlibException();
		return parseBlankSymbol(input);
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "EndSymbol")) {
		if(!features.count(FEATURES::END)) throw exception::AlibException();
		return parseEndSymbol(input);
	} else {
		throw sax::ParserException(sax::Token("LabeledSymbol, BlankSymbol, BottomOfTheStackSymbol, EndSymbol", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

Symbol SymbolFromXMLParser::parseLabeledSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol");
	label::Label data = alib::FromXMLParsers::labelParser.parse(input);
	popToken(input, sax::Token::TokenType::END_ELEMENT, "LabeledSymbol");
	return Symbol(LabeledSymbol(data));
}

Symbol SymbolFromXMLParser::parseBlankSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "BlankSymbol");
	return Symbol(BlankSymbol());
}

Symbol SymbolFromXMLParser::parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "BottomOfTheStackSymbol");
	return Symbol(BottomOfTheStackSymbol());
}

Symbol SymbolFromXMLParser::parseEndSymbol(std::list<sax::Token>& input) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "EndSymbol");
	popToken(input, sax::Token::TokenType::END_ELEMENT, "EndSymbol");
	return Symbol(EndSymbol());
}

bool SymbolFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "LabeledSymbol") || isToken(input, sax::Token::TokenType::START_ELEMENT, "BlankSymbol") || isToken(input, sax::Token::TokenType::START_ELEMENT, "BottomOfTheStackSymbol")) {
		return true;
	} else {
		return false;
	}
}

} /* namespace alphabet */
