/*
 * SymbolFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef SYMBOL_FROM_XML_PARSER_H_
#define SYMBOL_FROM_XML_PARSER_H_

#include "../FromXMLParser.hpp"
#include <vector>
#include <set>
#include "Symbol.h"
#include "SymbolFeatures.h"
#include "../sax/Token.h"
#include "../label/Label.h"

namespace string {

class StringFromXMLParser;

} /* namespace alphabet */

namespace regexp {

class RegExpFromXMLParser;

} /* namespace regexp */

namespace automaton {

class AutomatonFromXMLParser;

} /* namespace automaton */

namespace grammar {

class GrammarFromXMLParser;

} /* namespace grammar */

namespace alphabet {

/**
 * Parser used to get String from XML parsed into list of tokens.
 */
class SymbolFromXMLParser : public alib::FromXMLParser<Symbol, FEATURES> {
	virtual Symbol parse(std::list<sax::Token>& input) const;
	virtual Symbol parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;

	Symbol parseLabeledSymbol(std::list<sax::Token>& input) const;
	Symbol parseBlankSymbol(std::list<sax::Token>& input) const;
	Symbol parseBottomOfTheStackSymbol(std::list<sax::Token>& input) const;
	Symbol parseEndSymbol(std::list<sax::Token>& input) const;
public:
	virtual bool first(std::list<sax::Token>& input) const;

	friend class string::StringFromXMLParser;
	friend class regexp::RegExpFromXMLParser;
	friend class automaton::AutomatonFromXMLParser;
	friend class grammar::GrammarFromXMLParser;
};

} /* namespace alphabet */

#endif /* SYMBOL_FROM_XML_PARSER_H_ */
