/*
 * Automaton.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Martin Zak
 */

#ifndef AUTOMATON_H_
#define AUTOMATON_H_

#include "../std/visitor.hpp"
#include "AutomatonBase.h"

#include "common/State.h"
#include <set>

namespace automaton {

/**
 * Wrapper around automata.
 */
class Automaton {
protected:
	AutomatonBase* automaton;
public:
	explicit Automaton(const AutomatonBase& automaton);
	explicit Automaton(AutomatonBase&& automaton);
	Automaton(const Automaton& other);
	Automaton(Automaton&&) noexcept;
	Automaton& operator=(const Automaton& other);
	Automaton& operator=(Automaton&& other) noexcept;
	virtual ~Automaton() noexcept;

	const AutomatonBase& getAutomaton() const;
	AutomatonBase& getAutomaton();

	void setAutomaton(const AutomatonBase& automaton);
	void setAutomaton(AutomatonBase&& automaton);
	
	bool operator!=(const Automaton& other) const;

	bool operator==(const Automaton& other) const;

	friend std::ostream& operator<<(std::ostream& os, const Automaton& automaton);

	/**
	 * Creates and adds unique state to automaton. If given state name is
	 * already used, appends apostrophe or integer suffix
	 * @param name name of the state
	 * @throws AutomatonException if state could not be created
	 * @return created state
	 */
	static State createUniqueState(const State& base, const std::set<State>& other);
};

} /* namespace automaton */

#endif /* AUTOMATON_H_ */

