#ifndef AUTOMATON_PRINTER_H_
#define AUTOMATON_PRINTER_H_

#include <sstream>
#include "../Automaton.h"
#include "DFA.h"
#include "NFA.h"
#include "EpsilonNFA.h"

namespace automaton {

class FiniteAutomatonToStringComposer : public AutomatonBase::const_visitor_type {
	void Visit(void*, const Automaton& automaton) const;
	void Visit(void*, const UnknownAutomaton& automaton) const;
	void Visit(void*, const EpsilonNFA& automaton) const;
	void Visit(void*, const NFA& automaton) const;
	void Visit(void*, const DFA& automaton) const;
	void Visit(void*, const ExtendedNFA& automaton) const;
	void Visit(void*, const CompactNFA& automaton) const;
	void Visit(void*, const NPDA& automaton) const;
	void Visit(void*, const SinglePopNPDA& automaton) const;
	void Visit(void*, const OneTapeDTM& automaton) const;

	void composeTransitionsFromState(std::stringstream& out, const DFA& automaton, const State& from) const;
	void composeTransitionsFromState(std::stringstream& out, const NFA& automaton, const State& from) const;
	void composeTransitionsFromState(std::stringstream& out, const EpsilonNFA& automaton, const State& from) const;

public:
	/**
	 * Prints XML representation of UnknownAutomaton to the output stream.
	 * @param automaton automaton to print
	 * @return list of xml tokens representing the automaton
	 */
	std::string compose(const AutomatonBase& automaton) const;

	std::string compose(const Automaton& automaton) const;
	
	std::string compose(const DFA& automaton) const;
	std::string compose(const NFA& automaton) const;
	std::string compose(const EpsilonNFA& automaton) const;
};

} /* namespace automaton */

#endif /* AUTOMATON_PRINTER_H_ */
