/*
 * States.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "States.h"
#include <set>
#include "../AutomatonException.h"
#include <algorithm>

namespace automaton {

States::~States() {

}

bool States::addState(const State& state) {
	return states.insert(state).second;
}

void States::setStates(const std::set<State>& newStates) {
	std::set<State> removed;
	std::set_difference(states.begin(), states.end(), newStates.begin(), newStates.end(), std::inserter(removed, removed.end()));

	std::set<State> added;
	std::set_difference(newStates.begin(), newStates.end(), states.begin(), states.end(), std::inserter(added, added.end()));
	
	for(const State& removedState : removed) {
		removeState(removedState);
	}

	for(const State& addedState : added) {
		addState(addedState);
	}
}

const std::set<State>& States::getStates() const {
	return states;
}

bool States::addFinalState(const State& state) {
	if (states.find(state) == states.end()) {
		throw AutomatonException("State " + (std::string) state.getName() + " cannot be set as final state. It is not present in the automaton.");
	}

	return finalStates.insert(state).second;
}

void States::setFinalStates(const std::set<State>& newFinalStates) {
	std::set<State> removed;
	std::set_difference(finalStates.begin(), finalStates.end(), newFinalStates.begin(), newFinalStates.end(), std::inserter(removed, removed.end()));

	std::set<State> added;
	std::set_difference(newFinalStates.begin(), newFinalStates.end(), finalStates.begin(), finalStates.end(), std::inserter(added, added.end()));
	
	for(const State& removedState : removed) {
		removeFinalState(removedState);
	}

	for(const State& addedState : added) {
		addFinalState(addedState);
	}
}

bool States::removeFinalState(const State& state) {
	return finalStates.erase(state);
}

const std::set<State>& States::getFinalStates() const {
	return finalStates;
}

} /* namespace automaton */


