/*
 * AutomatonFactory.cpp
 *
 *  Created on: Apr 23, 2013
 *      Author: Martin Zak
 */

#include "AutomatonFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace automaton {

Automaton AutomatonFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

Automaton AutomatonFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

Automaton AutomatonFactory::fromStdin() {
	return AutomatonFactory::fromStream(std::cin);
}

Automaton AutomatonFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return AutomatonFactory::fromString(input);
}

Automaton AutomatonFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::automatonParser.parseValue(tokens);
}

void AutomatonFactory::toFile(const Automaton& automaton, const std::string& filename) {
	toFile(automaton.getAutomaton(), filename);
}

std::string AutomatonFactory::toString(const Automaton& automaton) {
	return toString(automaton.getAutomaton());
}

void AutomatonFactory::toStdout(const Automaton& automaton) {
	return AutomatonFactory::toStdout(automaton.getAutomaton());
}

void AutomatonFactory::toStream(const Automaton& automaton, std::ostream& out) {
	toStream(automaton.getAutomaton(), out);
}

void AutomatonFactory::toFile(const AutomatonBase& automaton, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(automaton);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string AutomatonFactory::toString(const AutomatonBase& automaton) {
	std::list<sax::Token> tokens = toTokens(automaton);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void AutomatonFactory::toStdout(const AutomatonBase& automaton) {
	return AutomatonFactory::toStream(automaton, std::cout);
}

void AutomatonFactory::toStream(const AutomatonBase& automaton, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(automaton);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> AutomatonFactory::toTokens(const AutomatonBase& automaton) {
	return alib::ToXMLComposers::automatonComposer.compose(automaton);
}

} /* namespace automaton */

