/*
 * AutomatonFactory.h
 *
 *  Created on: Apr 23, 2013
 *      Author: Martin Zak
 */

#ifndef AUTOMATON_FACTORY_H_
#define AUTOMATON_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../automaton/Automaton.h"

namespace automaton {

/**
 * Builds specific automatons from Automaton*.
 */
class AutomatonFactory {
public:
	/**
	 * Parses the XML in file and returns Automaton*.
	 * @param filename path to the file
	 * @return Automaton*
	 */
	static Automaton fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns Automaton*.
	 * @param str string containing the XML
	 * @return Automaton*
	 */
	static Automaton fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns Automaton*.
	 * @return Automaton*
	 */
	static Automaton fromStdin();
	
	/**
	 * Parses the XML from stream and returns Automaton*.
	 * @return Automaton*
	 */
	static Automaton fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the RegExp.
	 * @param filename path to the file
	 * @return RegExp
	 */
	static void toFile(const Automaton&, const std::string& filename);

	/**
	 * Parses the XML and returns the RegExp.
	 * @param str string containing the XML
	 * @return RegExp
	 */
	static std::string toString(const Automaton&);
	
	/**
	 * Parses the XML from stdin and returns the RegExp.
	 * @return RegExp
	 */
	static void toStdout(const Automaton&);
	
	/**
	 * Parses the XML from stream and returns the RegExp.
	 * @return RegExp
	 */
	static void toStream(const Automaton&, std::ostream& out);
	
	/**
	 * Parses the XML in file and returns the RegExp.
	 * @param filename path to the file
	 * @return RegExp
	 */
	static void toFile(const AutomatonBase&, const std::string& filename);

	/**
	 * Parses the XML and returns the RegExp.
	 * @param str string containing the XML
	 * @return RegExp
	 */
	static std::string toString(const AutomatonBase&);
	
	/**
	 * Parses the XML from stdin and returns the RegExp.
	 * @return RegExp
	 */
	static void toStdout(const AutomatonBase&);
	
	/**
	 * Parses the XML from stream and returns the RegExp.
	 * @return RegExp
	 */
	static void toStream(const AutomatonBase&, std::ostream& out);
	
	/**
	 * Parses the Automaton* from the list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed Automaton*
	 */
	static Automaton fromTokens(std::list<sax::Token> tokens);

	/**
	 * Parses the RegExp from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed RegExp
	 */
	static std::list<sax::Token> toTokens(const AutomatonBase&);
};

} /* namespace automaton */

#endif /* AUTOMATON_FACTORY_H_ */

