/*
 * ExceptionFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#include "ExceptionFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace exception {

AlibException ExceptionFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

AlibException ExceptionFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

AlibException ExceptionFactory::fromStdin() {
	return ExceptionFactory::fromStream(std::cin);
}

AlibException ExceptionFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return ExceptionFactory::fromString(input);
}

AlibException ExceptionFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::exceptionParser.parseValue(tokens);
}

void ExceptionFactory::toFile(const AlibException& exception, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(exception);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string ExceptionFactory::toString(const AlibException& exception) {
	std::list<sax::Token> tokens = toTokens(exception);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void ExceptionFactory::toStdout(const AlibException& exception) {
	return ExceptionFactory::toStream(exception, std::cout);
}

void ExceptionFactory::toStream(const AlibException& exception, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(exception);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> ExceptionFactory::toTokens(const AlibException& exception) {
	return alib::ToXMLComposers::exceptionComposer.compose(exception);
}

} /* namespace exception */
