/*
 * AlibExceptionFactory.h
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#ifndef EXCEPTION_FACTORY_H_
#define EXCEPTION_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../exception/AlibException.h"

namespace exception {

/**
 * String builder.
 */
class ExceptionFactory {
public:
	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static AlibException fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static AlibException fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static AlibException fromStdin();
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static AlibException fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the String.
	 * @param filename path to the file
	 * @return String
	 */
	static void toFile(const AlibException&, const std::string& filename);

	/**
	 * Parses the XML and returns the String.
	 * @param str string containing the XML
	 * @return String
	 */
	static std::string toString(const AlibException&);
	
	/**
	 * Parses the XML from stdin and returns the String.
	 * @return String
	 */
	static void toStdout(const AlibException&);
	
	/**
	 * Parses the XML from stream and returns the String.
	 * @return String
	 */
	static void toStream(const AlibException&, std::ostream& out);

	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static AlibException fromTokens(std::list<sax::Token> tokens);

	/**
	 * Parses the String from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed String
	 */
	static std::list<sax::Token> toTokens(const AlibException&);
};

} /* namespace exception */

#endif /* EXCEPTION_FACTORY_H_ */
