/*
 * GrammarFactory.cpp
 *
 *  Created on: Apr 23, 2013
 *      Author: Martin Zak
 */

#include "GrammarFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace grammar {

Grammar GrammarFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

Grammar GrammarFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

Grammar GrammarFactory::fromStdin() {
	return GrammarFactory::fromStream(std::cin);
}

Grammar GrammarFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return GrammarFactory::fromString(input);
}

Grammar GrammarFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::grammarParser.parseValue(tokens);
}

void GrammarFactory::toFile(const Grammar& grammar, const std::string& filename) {
	toFile(grammar.getGrammar(), filename);
}

std::string GrammarFactory::toString(const Grammar& grammar) {
	return toString(grammar.getGrammar());
}

void GrammarFactory::toStdout(const Grammar& grammar) {
	return GrammarFactory::toStdout(grammar.getGrammar());
}

void GrammarFactory::toStream(const Grammar& grammar, std::ostream& out) {
	toStream(grammar.getGrammar(), out);
}

void GrammarFactory::toFile(const GrammarBase& grammar, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(grammar);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string GrammarFactory::toString(const GrammarBase& grammar) {
	std::list<sax::Token> tokens = toTokens(grammar);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void GrammarFactory::toStdout(const GrammarBase& grammar) {
	return GrammarFactory::toStream(grammar, std::cout);
}

void GrammarFactory::toStream(const GrammarBase& grammar, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(grammar);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> GrammarFactory::toTokens(const GrammarBase& grammar) {
	return alib::ToXMLComposers::grammarComposer.compose(grammar);
}

} /* namespace grammar */

