/*
 * GrammarFactory.h
 *
 *  Created on: Apr 23, 2013
 *      Author: Martin Zak
 */

#ifndef GRAMMAR_FACTORY_H_
#define GRAMMAR_FACTORY_H_

#include <string>
#include <list>
#include "../sax/Token.h"
#include "../grammar/Grammar.h"

namespace grammar {

/**
 * Builds specific grammars from Grammar*.
 */
class GrammarFactory {
public:
	/**
	 * Parses the XML in file and returns Grammar*.
	 * @param filename path to the file
	 * @return Grammar*
	 */
	static Grammar fromFile(const std::string& filename);

	/**
	 * Parses the XML and returns Grammar*.
	 * @param str string containing the XML
	 * @return Grammar*
	 */
	static Grammar fromString(const std::string& str);
	
	/**
	 * Parses the XML from stdin and returns Grammar*.
	 * @return Grammar*
	 */
	static Grammar fromStdin();
	
	/**
	 * Parses the XML from stream and returns Grammar*.
	 * @return Grammar*
	 */
	static Grammar fromStream(std::istream& in);

	/**
	 * Parses the XML in file and returns the Grammar.
	 * @param filename path to the file
	 * @return Grammar
	 */
	static void toFile(const Grammar&, const std::string& filename);

	/**
	 * Parses the XML and returns the Grammar.
	 * @param str string containing the XML
	 * @return Grammar
	 */
	static std::string toString(const Grammar&);
	
	/**
	 * Parses the XML from stdin and returns the Grammar.
	 * @return Grammar
	 */
	static void toStdout(const Grammar&);
	
	/**
	 * Parses the XML from stream and returns the Grammar.
	 * @return Grammar
	 */
	static void toStream(const Grammar&, std::ostream& out);
	
	/**
	 * Parses the XML in file and returns the Grammar.
	 * @param filename path to the file
	 * @return Grammar
	 */
	static void toFile(const GrammarBase&, const std::string& filename);

	/**
	 * Parses the XML and returns the Grammar.
	 * @param str string containing the XML
	 * @return Grammar
	 */
	static std::string toString(const GrammarBase&);
	
	/**
	 * Parses the XML from stdin and returns the Grammar.
	 * @return Grammar
	 */
	static void toStdout(const GrammarBase&);
	
	/**
	 * Parses the XML from stream and returns the Grammar.
	 * @return Grammar
	 */
	static void toStream(const GrammarBase&, std::ostream& out);
	
	/**
	 * Parses the Grammar* from the list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed Grammar*
	 */
	static Grammar fromTokens(std::list<sax::Token> tokens);

	/**
	 * Parses the Grammar from list of tokens.
	 * @param tokens XML represented as list of tokens
	 * @return parsed Grammar
	 */
	static std::list<sax::Token> toTokens(const GrammarBase&);
};

} /* namespace grammar */

#endif /* GRAMMAR_FACTORY_H_ */

