/*
 * LabelFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#include "LabelFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace label {

Label LabelFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

Label LabelFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

Label LabelFactory::fromStdin() {
	return LabelFactory::fromStream(std::cin);
}

Label LabelFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return LabelFactory::fromString(input);
}

Label LabelFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::labelParser.parseValue(tokens);
}

void LabelFactory::toFile(const Label& symbol, const std::string& filename) {
	std::list<sax::Token> tokens 	= toTokens(symbol);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string LabelFactory::toString(const Label& symbol) {
	std::list<sax::Token> tokens = toTokens(symbol);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void LabelFactory::toStdout(const Label& symbol) {
	return LabelFactory::toStream(symbol, std::cout);
}

void LabelFactory::toStream(const Label& symbol, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(symbol);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> LabelFactory::toTokens(const Label& symbol) {
	return alib::ToXMLComposers::labelComposer.compose(symbol);
}

} /* namespace label */
