/*
 * RegExpFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Martin Zak
 */

#include "RegExpFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace regexp {

RegExp RegExpFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

RegExp RegExpFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

RegExp RegExpFactory::fromStdin() {
	return RegExpFactory::fromStream(std::cin);
}

RegExp RegExpFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return RegExpFactory::fromString(input);
}

RegExp RegExpFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::regexpParser.parseValue(tokens);
}

void RegExpFactory::toFile(const RegExp& regexp, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(regexp);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string RegExpFactory::toString(const RegExp& regexp) {
	std::list<sax::Token> tokens = toTokens(regexp);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void RegExpFactory::toStdout(const RegExp& regexp) {
	return RegExpFactory::toStream(regexp, std::cout);
}

void RegExpFactory::toStream(const RegExp& regexp, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(regexp);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> RegExpFactory::toTokens(const RegExp& regexp) {
	return alib::ToXMLComposers::regexpComposer.compose(regexp);
}

} /* namespace regexp */
