/*
 * StringFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#include "StringFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace string {

String StringFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

String StringFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

String StringFactory::fromStdin() {
	return StringFactory::fromStream(std::cin);
}

String StringFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return StringFactory::fromString(input);
}

String StringFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::stringParser.parseValue(tokens);
}

void StringFactory::toFile(const String& string, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(string);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string StringFactory::toString(const String& string) {
	std::list<sax::Token> tokens = toTokens(string);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void StringFactory::toStdout(const String& string) {
	return StringFactory::toStream(string, std::cout);
}

void StringFactory::toStream(const String& string, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(string);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> StringFactory::toTokens(const String& string) {
	return alib::ToXMLComposers::stringComposer.compose(string);
}

} /* namespace string */
