/*
 * SymbolFactory.cpp
 *
 *  Created on: Jan 1, 2014
 *      Author: Jan Travnicek
 */

#include "SymbolFactory.h"

#include "../sax/SaxParseInterface.h"
#include "../sax/SaxComposeInterface.h"
#include "../FromXMLParsers.h"
#include "../ToXMLComposers.h"

namespace alphabet {

Symbol SymbolFactory::fromFile(const std::string& filename) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseFile(filename, tokens);
	return fromTokens(tokens);
}

Symbol SymbolFactory::fromString(const std::string& str) {
	std::list<sax::Token> tokens;
	sax::SaxParseInterface::parseMemory(str, tokens);
	return fromTokens(tokens);
}

Symbol SymbolFactory::fromStdin() {
	return SymbolFactory::fromStream(std::cin);
}

Symbol SymbolFactory::fromStream(std::istream& in) {
	std::string input(std::istreambuf_iterator<char>(in), (std::istreambuf_iterator<char>()));
	return SymbolFactory::fromString(input);
}

Symbol SymbolFactory::fromTokens(std::list<sax::Token> tokens) {
	return alib::FromXMLParsers::symbolParser.parseValue(tokens);
}

void SymbolFactory::toFile(const Symbol& symbol, const std::string& filename) {
	std::list<sax::Token> tokens = toTokens(symbol);
	sax::SaxComposeInterface::printFile(filename, tokens);
}

std::string SymbolFactory::toString(const Symbol& symbol) {
	std::list<sax::Token> tokens = toTokens(symbol);
	std::string str;
	sax::SaxComposeInterface::printMemory(str, tokens);
	return str;
}

void SymbolFactory::toStdout(const Symbol& symbol) {
	return SymbolFactory::toStream(symbol, std::cout);
}

void SymbolFactory::toStream(const Symbol& symbol, std::ostream& out) {
	std::list<sax::Token> tokens = toTokens(symbol);
	sax::SaxComposeInterface::printStream(out, tokens);
}

std::list<sax::Token> SymbolFactory::toTokens(const Symbol& symbol) {
	return alib::ToXMLComposers::symbolComposer.compose(symbol);
}

} /* namespace alphabet */
