/*
 * CFG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef CFG_H_
#define CFG_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Context free grammar. Type 2 in Chomsky hierarchy. Produces context free language.
 */
class CFG : public std::element<CFG, GrammarBase>, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> rules;
	bool generatesEpsilon;
public:
	CFG(const alphabet::Symbol& initialSymbol);

	CFG(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	const std::map<alphabet::Symbol, std::set<std::vector<alphabet::Symbol>>> getRules() const;

	bool removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	virtual bool operator==(const GrammarBase& other) const;

	virtual bool operator==(const CFG& other) const;

	virtual void operator>>(std::ostream& os) const;
};

} /* namespace grammar */

#endif /* CFG_H_ */
