/*
 * Grammar.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Martin Zak
 */

#include "Grammar.h"

namespace grammar {

Grammar::Grammar(const GrammarBase& grammar) : grammar(grammar.clone()) {

}

Grammar::Grammar(GrammarBase&& grammar) : grammar(std::move(grammar).plunder()) {

}

Grammar::Grammar(const Grammar& other) : grammar(other.getGrammar().clone()) {

}

Grammar::Grammar(Grammar&& other) noexcept : grammar(std::move(other.getGrammar()).plunder()) {
	other.grammar = NULL;
}

Grammar& Grammar::operator=(const Grammar& other) {
	if(this == &other) return *this;

	delete grammar;
	grammar = other.getGrammar().clone();

	return *this;
}

Grammar& Grammar::operator=(Grammar&& other) noexcept {
	std::swap(this->grammar, other.grammar);
	return *this;
}

Grammar::~Grammar() {
	delete grammar;
}

const GrammarBase& Grammar::getGrammar() const {
	return *grammar;
}

GrammarBase& Grammar::getGrammar() {
	return *grammar;
}

void Grammar::setGrammar(const GrammarBase& grammar) {
	delete this->grammar;
	this->grammar = grammar.clone();
}

void Grammar::setGrammar(GrammarBase&& grammar) {
	delete this->grammar;
	this->grammar = std::move(grammar).plunder();
}

bool Grammar::operator!=(const Grammar& other) const {
	return !(*this == other);
}

bool Grammar::operator==(const Grammar& other) const {
	return this->getGrammar() == other.getGrammar();
}

std::ostream& operator<<(std::ostream& os, const Grammar& grammar) {
	os << grammar.getGrammar();
	return os;
}

} /* namespace grammar */

