/*
 * GrammarBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef GRAMMAR_BASE_H_
#define GRAMMAR_BASE_H_

#include "../std/visitor.hpp"
#include <iostream>

namespace grammar {

class UnknownGrammar;
class LeftLG;
class LeftRG;
class RightLG;
class RightRG;
class LG;
class CFG;
class EpsilonFreeCFG;
class CNF;
class GNF;
class CSG;
class NonContractingGrammar;
class ContextPreservingUnrestrictedGrammar;
class UnrestrictedGrammar;

/**
 * Abstract base class for all automata.
 */
class GrammarBase : public std::elementBase<UnknownGrammar, LeftLG, LeftRG, RightLG, RightRG, LG, CFG, EpsilonFreeCFG, CNF, GNF, CSG, NonContractingGrammar, ContextPreservingUnrestrictedGrammar, UnrestrictedGrammar> {
public:
	virtual GrammarBase* clone() const = 0;

	virtual GrammarBase* plunder() && = 0;

	virtual ~GrammarBase() noexcept;

	virtual bool operator!=(const GrammarBase& other) const;

	virtual bool operator==(const UnknownGrammar& other) const;

	virtual bool operator==(const LeftLG& other) const;

	virtual bool operator==(const LeftRG& other) const;

	virtual bool operator==(const RightLG& other) const;

	virtual bool operator==(const RightRG& other) const;

	virtual bool operator==(const LG& other) const;

	virtual bool operator==(const CFG& other) const;

	virtual bool operator==(const EpsilonFreeCFG& other) const;

	virtual bool operator==(const CNF& other) const;
	
	virtual bool operator==(const GNF& other) const;
	
	virtual bool operator==(const CSG& other) const;
	
	virtual bool operator==(const NonContractingGrammar& other) const;
	
	virtual bool operator==(const UnrestrictedGrammar& other) const;
	
	virtual bool operator==(const ContextPreservingUnrestrictedGrammar& other) const;

	virtual bool operator==(const GrammarBase& other) const = 0;

	friend std::ostream& operator<<(std::ostream& os, const GrammarBase& grammar);

	virtual void operator>>(std::ostream&) const = 0;
};

} /* namespace grammar */

#endif /* GRAMMAR_BASE_H_ */

