/*
 * RightLG.h
 *
 *  Created on: Nov 17, 2013
 *      Author: Jan Travnicek
 */

#ifndef RIGHT_LG_H_
#define RIGHT_LG_H_

#include "../GrammarBase.h"
#include <map>
#include <vector>
#include "../../std/variant.hpp"
#include "../common/TerminalNonterminalAlphabetInitialSymbol.h"

namespace grammar {

/**
 * Right linear grammar in chomsky normal form. Type 3 in Chomsky hierarchy. Produces regular languages.
 */
class RightLG : public std::element<RightLG, GrammarBase>, public TerminalNonterminalAlphabetInitialSymbol {
	std::map<alphabet::Symbol, std::set<std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol> >> > rules;
public:
	RightLG(const alphabet::Symbol& initialSymbol);

	RightLG(const std::set<alphabet::Symbol>& nonTerminalSymbols, const std::set<alphabet::Symbol>& terminalSymbols, const alphabet::Symbol& initialSymbol);

	virtual GrammarBase* clone() const;

	virtual GrammarBase* plunder() &&;

	bool addRule(const alphabet::Symbol& leftHandSide, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol> >& rightHandSide);
	bool addRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);
	bool addRule(const alphabet::Symbol& leftHandSide, const std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>& rightHandSide);

	bool addRawRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);

	const std::map<alphabet::Symbol, std::set<std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol> >> > getRules() const;

	bool removeRule(const alphabet::Symbol& leftHandSide, const std::variant<std::vector<alphabet::Symbol>, std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol> >& rightHandSide);
	bool removeRule(const alphabet::Symbol& leftHandSide, const std::vector<alphabet::Symbol>& rightHandSide);
	bool removeRule(const alphabet::Symbol& leftHandSide, const std::pair<std::vector<alphabet::Symbol>, alphabet::Symbol>& rightHandSide);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	virtual bool operator==(const GrammarBase& other) const;

	virtual bool operator==(const RightLG& other) const;

	virtual void operator>>(std::ostream& os) const;
};

} /* namespace grammar */

#endif /* RIGHT_LG_H_ */
