/*
 * UnknownGrammar.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "UnknownGrammar.h"
#include "GrammarException.h"
#include "../std/set.hpp"

namespace grammar {

UnknownGrammar::UnknownGrammar() : initialSymbol(NULL) {

}

GrammarBase* UnknownGrammar::clone() const {
	return new UnknownGrammar(*this);
}

GrammarBase* UnknownGrammar::plunder() && {
	return new UnknownGrammar(std::move(*this));
}

bool UnknownGrammar::addTerminalSymbol(const alphabet::Symbol& symbol) {
	return terminalAlphabet.insert(symbol).second;
}

bool UnknownGrammar::removeTerminalSymbol(const alphabet::Symbol& symbol) {
	return terminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& UnknownGrammar::getTerminalAlphabet() const {
	return terminalAlphabet;
}

void UnknownGrammar::setTerminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	terminalAlphabet = alphabet;
}

bool UnknownGrammar::addNonterminalSymbol(const alphabet::Symbol& symbol) {
	return nonterminalAlphabet.insert(symbol).second;
}

bool UnknownGrammar::removeNonterminalSymbol(const alphabet::Symbol& symbol) {
	return nonterminalAlphabet.insert(symbol).second;
}

const std::set<alphabet::Symbol>& UnknownGrammar::getNonterminalAlphabet() const {
	return nonterminalAlphabet;
}

void UnknownGrammar::setNonterminalAlphabet(const std::set<alphabet::Symbol>& alphabet) {
	nonterminalAlphabet = alphabet;
}

void UnknownGrammar::setInitialSymbol(const alphabet::Symbol& symbol) {
	delete initialSymbol;
	initialSymbol = new alphabet::Symbol(symbol);
}

bool UnknownGrammar::hasInitialSymbol() const {
	return initialSymbol != NULL;
}

void UnknownGrammar::clearInitialSymbol() {
	delete initialSymbol;
	initialSymbol = NULL;
}

const alphabet::Symbol& UnknownGrammar::getInitialSymbol() const {
	return *initialSymbol;
}

int UnknownGrammar::addRule(const UnknownRule& rule) {
	return rules.insert(rule).second;
}

int UnknownGrammar::removeRule(const UnknownRule& rule) {
	return rules.erase(rule);
}

const std::set<UnknownRule>& UnknownGrammar::getRules() const {
	return rules;
}

bool UnknownGrammar::operator==(const GrammarBase& other) const {
	return other == *this;
}

bool UnknownGrammar::operator==(const UnknownGrammar& other) const {
	return terminalAlphabet == other.terminalAlphabet
		&& nonterminalAlphabet == other.nonterminalAlphabet
		&& (initialSymbol == other.initialSymbol || (initialSymbol != NULL && other.initialSymbol != NULL && *initialSymbol == *other.initialSymbol))
		&& rules == other.rules;
}

void UnknownGrammar::operator>>(std::ostream& out) const {
	out << "(UnknownGrammar"
		<< " nonterminalAlphabet = " << nonterminalAlphabet
		<< " terminalAlphabet = " << terminalAlphabet
		<< " initialSymbol = ";
	if(initialSymbol == NULL) out << "NULL"; else out << *initialSymbol;
	out << " rules = " << rules
		<< ")";
}

} /* namespace grammar */
