/*
 * UnknownGrammar.h
 *
 *  Created on: Nov 3, 2013
 *      Author: Jan Travnicek
 */

#ifndef UNKNOWN_GRAMMAR_H_
#define UNKNOWN_GRAMMAR_H_

#include "GrammarBase.h"
#include "UnknownRule.h"
#include <set>

namespace grammar {

/**
 * Class representing unknown grammar which was parsed from the XML.
 */
class UnknownGrammar : public std::element<UnknownGrammar, GrammarBase> {
	std::set<alphabet::Symbol> terminalAlphabet;
	std::set<alphabet::Symbol> nonterminalAlphabet;
	std::set<UnknownRule> rules;
	alphabet::Symbol* initialSymbol;
public:
	UnknownGrammar();
	
	virtual GrammarBase* clone() const;
	
	virtual GrammarBase* plunder() &&;

	bool addTerminalSymbol(const alphabet::Symbol& symbol);

	bool removeTerminalSymbol(const alphabet::Symbol& symbol);

	const std::set<alphabet::Symbol>& getTerminalAlphabet() const;

	void setTerminalAlphabet(const std::set<alphabet::Symbol>& alphabet);

	bool addNonterminalSymbol(const alphabet::Symbol& symbol);

	bool removeNonterminalSymbol(const alphabet::Symbol& symbol);

	const std::set<alphabet::Symbol>& getNonterminalAlphabet() const;

	void setNonterminalAlphabet(const std::set<alphabet::Symbol>& alphabet);

	void setInitialSymbol(const alphabet::Symbol& symbol);

	bool hasInitialSymbol() const;

	void clearInitialSymbol();

	const alphabet::Symbol& getInitialSymbol() const;

	int addRule(const UnknownRule& rule);

	int removeRule(const UnknownRule& rule);

	const std::set<UnknownRule>& getRules() const;

	virtual bool operator==(const GrammarBase& other) const;

	virtual bool operator==(const UnknownGrammar& other) const;

	virtual void operator>>(std::ostream& out) const;

};

} /* namespace grammar */
#endif /* UNKNOWN_GRAMMAR_H_ */
