/*
 * UnknownRule.cpp
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#include "UnknownRule.h"
#include <algorithm>
#include "../std/vector.hpp"

namespace grammar {

UnknownRule::UnknownRule() {
}

UnknownRule::UnknownRule(const std::vector<alphabet::Symbol>& leftSide, const std::vector<alphabet::Symbol>& rightSide) :
		leftSide(leftSide), rightSide(rightSide) {
}

void grammar::UnknownRule::setLeftHandSide(const std::vector<alphabet::Symbol>& leftSide) {
	this->leftSide = leftSide;
}

void grammar::UnknownRule::setRightHandSide(const std::vector<alphabet::Symbol>& rightSide) {
	this->rightSide = rightSide;
}

const std::vector<alphabet::Symbol>& grammar::UnknownRule::getLeftHandSide() const {
	return leftSide;
}

const std::vector<alphabet::Symbol>& grammar::UnknownRule::getRightHandSide() const {
	return rightSide;
}

std::string UnknownRule::toString() const {
	std::string output;
	output += "[";
	for(auto const& symbol : leftSide) {
		output += " " + (std::string) symbol.getSymbol();
	}
	output += " -> ";
	for(auto const& symbol : rightSide) {
		output += " " + (std::string) symbol.getSymbol();
	}

	output += "]";
	return output;
}

bool UnknownRule::operator <(const UnknownRule& other) const {
	if(leftSide < other.leftSide) return true;
	if(leftSide == other.leftSide && rightSide < other.rightSide) return true;
	return false;
}

bool UnknownRule::operator ==(const UnknownRule& other) const {
	return leftSide == other.leftSide && rightSide == other.rightSide;
}

bool UnknownRule::operator !=(const UnknownRule& other) const {
	return !(*this == other);
}

std::ostream& operator<<(std::ostream& out, const UnknownRule& rule) {
	out << " leftSide = " << rule.leftSide;
	out << " rightSide = " << rule.rightSide;
	return out;
}

} /* namespace grammar */
