/*
 * TerminalNonterminalAlphabetInitialSymbol.h
 *
 *  Created on: Nov 2, 2013
 *      Author: Martin Zak
 */

#ifndef TERMINAL_NONTERMINAL_ALPHABET_INITIAL_SYMBOL_H_
#define TERMINAL_NONTERMINAL_ALPHABET_INITIAL_SYMBOL_H_

#include <set>
#include <list>

#include "../../alphabet/Symbol.h"

namespace grammar {

/**
 * Abstract class representing grammar. Implements methods for manipulation
 * with terminal and nonterminal symbols. Only method, which is not implemented
 * is grammar specific isValidRule.
 */
class TerminalNonterminalAlphabetInitialSymbol {
protected:
	std::set<alphabet::Symbol> terminalAlphabet;
	std::set<alphabet::Symbol> nonterminalAlphabet;
	alphabet::Symbol initialSymbol;

public:
	TerminalNonterminalAlphabetInitialSymbol(const alphabet::Symbol& initialSymbol);

	/**
	 * Adds terminal symbol to the terminal alphabet.
	 * @param symbol Symbol to add
	 * @throws GrammarException when symbol is already a nonterminal symbol
	 */
	bool addTerminalSymbol(const alphabet::Symbol& symbol);

	/**
	 * Removes terminal symbol from the terminal alphabet.
	 * @param symbol Symbol to remove
	 * @throws GrammarException when symbol is used in rule
	 */
	virtual bool removeTerminalSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * return set of terminal symbols
	 */
	const std::set<alphabet::Symbol>& getTerminalAlphabet() const;

	/**
	 * Sets the terminal alphabet
	 */
	void setTerminalAlphabet(const std::set<alphabet::Symbol>& alphabet);

	/**
	 * Adds nonterminal symbol to the alphabet of nonterminals.
	 * @param symbol Symbol to add
	 * @throws GrammarException when symbol is already a terminal
	 */
	bool addNonterminalSymbol(const alphabet::Symbol& symbol);

	/**
	 * Removes nonterminal symbol from the grammar.
	 * @param symbol Symbol to remove
	 * @throws GrammarException when symbol is used in rule
	 */
	virtual bool removeNonterminalSymbol(const alphabet::Symbol& symbol) = 0;

	/**
	 * @return set of nonterminal symbols
	 */
	const std::set<alphabet::Symbol>& getNonterminalAlphabet() const;

	/**
	 * Sets the terminal alphabet
	 */
	void setNonterminalAlphabet(const std::set<alphabet::Symbol>& alphabet);

	/**
	 * @return start symbol
	 */
	const alphabet::Symbol& getInitialSymbol() const;

	/**
	 * Sets the initial symbol.
	 * @param symbol nonterminal Symbol
	 * @throws GrammarException when symbol isn't a nonterminal symbol
	 */
	void setInitialSymbol(const alphabet::Symbol& symbol);

};

} /* namespace grammar */

#endif /* TERMINAL_NONTERMINAL_ALPHABET_INITIAL_SYMBOL_H_ */
