/*
 * Label.cpp
 *
 *  Created on: Apr 16, 2013
 *      Author: Jan Travnicek
 */

#include "Label.h"

namespace label {

Label::Label(const LabelBase& label) : label(label.clone()) {

}

Label::Label(LabelBase&& label) : label(std::move(label).plunder()) {

}

Label::Label(const Label& other) : label(other.getLabel().clone()) {

}

Label::Label(Label&& other) noexcept : label(other.label) {
	other.label = NULL;
}

Label& Label::operator=(const Label& other) {
	if(this == &other) return *this;

	delete label;
	label = other.getLabel().clone();

	return *this;
}

Label& Label::operator=(Label&& other) noexcept {
	std::swap(this->label, other.label);
	return *this;
}

Label::~Label() {
	delete label;
}

const LabelBase& Label::getLabel() const {
	return *label;
}

LabelBase& Label::getLabel() {
	return *label;
}

void Label::setLabel(const LabelBase& label) {
	delete this->label;
	this->label = label.clone();
}

void Label::setLabel(LabelBase&& label) {
	delete this->label;
	this->label = std::move(label).plunder();
}

bool Label::operator!=(const Label& other) const {
	return !(*this == other);
}

bool Label::operator<(const Label& other) const {
	return *label < *other.label;
}

bool Label::operator==(const Label& other) const {
	return *label == *other.label;
}

std::ostream& operator<<(std::ostream& os, const Label& label) {
	os << label.getLabel();
	return os;
}

Label::operator std::string () const {
	return (std::string) *label;
}

} /* namespace label */

