/*
 * LabelBase.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef LABEL_BASE_H_
#define LABEL_BASE_H_

#include "../std/visitor.hpp"
#include <iostream>

namespace label {

class StringLabel;
class IntegerLabel;
class CharacterLabel;

/**
 * Abstract base class for all automata.
 */
class LabelBase : public std::elementBase<StringLabel, IntegerLabel, CharacterLabel> {
public:
	virtual LabelBase* clone() const = 0;

	virtual LabelBase* plunder() && = 0;

	virtual ~LabelBase() noexcept;
	
	virtual bool operator<(const LabelBase& other) const = 0;
	
	virtual bool operator<(const StringLabel& other) const;
	
	virtual bool operator<(const IntegerLabel& other) const;
	
	virtual bool operator<(const CharacterLabel& other) const;
	
	virtual bool operator>(const LabelBase& other) const = 0;

	
	bool operator>=(const LabelBase& other) const;

	bool operator<=(const LabelBase& other) const;

	bool operator!=(const LabelBase& other) const;

	
	virtual bool operator==(const LabelBase& other) const = 0;

	virtual bool operator==(const StringLabel& other) const;

	virtual bool operator==(const IntegerLabel& other) const;

	virtual bool operator==(const CharacterLabel& other) const;

	friend std::ostream& operator<<(std::ostream& os, const LabelBase& label);

	virtual void operator>>(std::ostream&) const = 0;

	virtual operator std::string() const = 0;
};

} /* namespace label */

#endif /* LABEL_BASE_H_ */

