#include "LabelFromStringLexer.h"

namespace label {

LabelFromStringLexer::LabelFromStringLexer(std::stringstream& in) : m_In(in) {
	m_Current.type = TokenType::ERROR;
	m_Current.value = "";
}

LabelFromStringLexer& LabelFromStringLexer::next() {
	if(m_Current.type == TokenType::TEOF) return *this;
	char character;
	m_Current.value = "";

	std::streampos pos = m_In.tellg();
L0:
	character = m_In.get();
	if(m_In.eof()) {
		m_In.seekg(pos);
		m_Current.type = TokenType::TEOF;
		return *this;
	} else if(character == ' ' || character == '\n' || character == '\t') {
		goto L0;
	} else if((character >= 'a' && character <= 'z') || (character >= 'A' && character <= 'Z')) {
		m_Current.type = TokenType::CHAR;
		m_Current.value += character;
		return *this;
	} else if(character >= '0' && character <= '9') {
		m_Current.type = TokenType::INTEGER;
		m_Current.value += character;
		goto L1;
	} else if(character == '\'') {
		m_Current.type = TokenType::STRING;
		goto L3;
	} else {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	}
L1:
	character = m_In.get();
	if(m_In.eof()) {
		return *this;
	} else if(character >= '0' && character <= '9') {
		m_Current.value += character;
		goto L1;
	} else {
		m_In.unget();
		return *this;
	}
L3:
	character = m_In.get();
	if(m_In.eof()) {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	} else if(character == '\'') {
		return *this;
	} else if(character == '\\') {
		goto L4;
	} else {
		m_Current.value += character;
		goto L3;
	}
L4:
	character = m_In.get();
	if(m_In.eof()) {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	} else if(character == '\'' || character == '\\') {
		m_Current.value += character;
		goto L3;
	} else {
		m_In.seekg(pos);
		m_Current.type = TokenType::ERROR;
		return *this;
	}
}

LabelFromStringLexer::Token LabelFromStringLexer::token() {
	return m_Current;
}

} /* namespace label */
