/*
 * LabelFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelFromXMLParser.h"
#include "../sax/ParserException.h"
#include "StringLabel.h"
#include "IntegerLabel.h"
#include "CharacterLabel.h"
#include <algorithm>

namespace label {

Label LabelFromXMLParser::parse(std::list<sax::Token>& input) const {
	return parse(input, std::set<FEATURES>({FEATURES::STRING, FEATURES::CHAR, FEATURES::INTEGER}));
}

Label LabelFromXMLParser::parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "IntegerLabel")) {
		if(!features.count(FEATURES::INTEGER)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "IntegerLabel");
		Label data(IntegerLabel(std::stoi(popTokenData(input, sax::Token::TokenType::CHARACTER))));
		popToken(input, sax::Token::TokenType::END_ELEMENT, "IntegerLabel");
		return data;
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "StringLabel")) {
		if(!features.count(FEATURES::STRING)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "StringLabel");
		Label data(StringLabel(popTokenData(input, sax::Token::TokenType::CHARACTER)));
		popToken(input, sax::Token::TokenType::END_ELEMENT, "StringLabel");
		return data;
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "CharacterLabel")) {
		if(!features.count(FEATURES::CHAR)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "CharacterLabel");
		Label data(CharacterLabel(popTokenData(input, sax::Token::TokenType::CHARACTER)[0]));
		popToken(input, sax::Token::TokenType::END_ELEMENT, "CharacterLabel");
		return data;
	} else {
		throw sax::ParserException(sax::Token("IntegerLabel, StringLabel, CharacterLabel", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

bool LabelFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "IntegerLabel") || isToken(input, sax::Token::TokenType::START_ELEMENT, "StringLabel") || isToken(input, sax::Token::TokenType::START_ELEMENT, "CharacterLabel")) {
		return true;
	} else {
		return false;
	}
}

} /* namespace label */
