/*
 * LabelToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LabelToStringComposer.h"
#include <algorithm>
#include "IntegerLabel.h"
#include "StringLabel.h"
#include "CharacterLabel.h"

namespace label {

void LabelToStringComposer::Visit(void* userData, const label::StringLabel& label) {
	std::stringstream &out = *((std::stringstream*) userData);

	auto replace = [](std::string& str, const std::string& what, const std::string& with) {
		size_t index = 0;
		while((index = str.find(what, index)) != std::string::npos) {
			str.replace(index, what.length(), with);
			index += with.length();
		}
	};
	
	std::string tmp = label.getData();
	replace(tmp, "'", "\\'" );
	out << '\'' << tmp << '\'';
}

void LabelToStringComposer::Visit(void* userData, const CharacterLabel& symbol) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << symbol.getData();
}

void LabelToStringComposer::Visit(void* userData, const IntegerLabel& symbol) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << symbol.getData();
}

void LabelToStringComposer::Visit(void* userData, const Label& symbol) {
	symbol.getLabel().Accept(userData, *this);

}

std::string LabelToStringComposer::compose(const Label& symbol) {
	std::stringstream out;
	Visit((void*) &out, symbol);
	return std::move(out).str();
}

} /* namespace label */
