/*
 * Alternation.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef ALTERNATION_H_
#define ALTERNATION_H_

#include <vector>
#include "RegExpElement.h"

namespace regexp {

/**
 * Represents alternation operator in the regular expression. Contains list of RegExpElement
 * as operands of the operator.
 */
class Alternation: public std::element<Alternation, RegExpElement> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	/**
	 * @copydoc RegExpElement::plunder() const
	 */
	virtual RegExpElement* plunder() &&;

	std::vector<RegExpElement*> elements;

	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
	
public:
	explicit Alternation(RegExpElement&& left, RegExpElement&& right);
	explicit Alternation(const RegExpElement& left, const RegExpElement& right);
	
	Alternation(const Alternation& other);
	Alternation(Alternation&& other) noexcept;
	Alternation& operator =(const Alternation& other);
	Alternation& operator =(Alternation&& other);
	virtual ~Alternation() noexcept;
	
	/**
	 * @return elements
	 */
	const std::vector<const RegExpElement*> & getElements() const;
	
	/**
	 * @return elements
	 */
	const std::vector<RegExpElement*> & getElements();

	/**
	 * @param element to append
	 */
	void appendElement(const RegExpElement& element);

	/**
	 * @param element to append
	 */
	void appendElement(RegExpElement&& element);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Alternation&) const;
	virtual bool operator==(const Alternation&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* ALTERNATION_H_ */
