/*
 * Concatenation.h
 *
 *  Created on: Nov 27, 2013
 *      Author: Martin Zak
 */

#ifndef CONCATENATION_H_
#define CONCATENATION_H_

#include <vector>
#include "RegExpElement.h"

namespace regexp {

/**
 * Represents concatenation operator in the regular expression. Contains list of RegExpElement
 * as operands of the operator.
 */
class Concatenation: public std::element<Concatenation, RegExpElement> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	std::vector<RegExpElement*> elements;
	
	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	explicit Concatenation(RegExpElement&& left, RegExpElement&& right);
	explicit Concatenation(const RegExpElement& left, const RegExpElement& right);
	
	Concatenation(const Concatenation& other);
	Concatenation(Concatenation&& other) noexcept;
	Concatenation& operator =(const Concatenation& other);
	Concatenation& operator =(Concatenation&& other);
	virtual ~Concatenation() noexcept;
	
	/**
	 * @return elements
	 */
	const std::vector<const RegExpElement*> & getElements() const;

	/**
	 * @return elements
	 */
	const std::vector<RegExpElement*> & getElements();

	/**
	 * @param element to append
	 */
	void appendElement(const RegExpElement& element);
	
	void appendElement(RegExpElement&& element);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Concatenation&) const;
	virtual bool operator==(const Concatenation&) const;

	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* CONCATENATION_H_ */
