/*
 * Iteration.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef ITERATION_H_
#define ITERATION_H_

#include "RegExpElement.h"
#include "RegExpEmpty.h"

namespace regexp {

/**
 * Represents iteration operator in the regular expression. Contains one RegExpElement
 * as operand.
 */
class Iteration: public std::element<Iteration, RegExpElement> {
protected:
	RegExpElement* element;

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;

	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
	
public:
	explicit Iteration(RegExpElement&&);
	explicit Iteration(const RegExpElement&);
	
	Iteration(const Iteration& other);
	Iteration(Iteration&& other) noexcept;
	Iteration& operator =(const Iteration& other);
	Iteration& operator =(Iteration&& other);
	virtual ~Iteration() noexcept;

	/**
	 * @return element
	 */
	const RegExpElement & getElement() const;
	
	/**
	 * @return element
	 */
	RegExpElement & getElement();
	
	/**
	 * @param element to iterate
	 */
	void setElement(const RegExpElement& element);
	
	void setElement(RegExpElement&& element);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const Iteration&) const;
	virtual bool operator==(const Iteration&) const;

	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* ITERATION_H_ */
