/*
 * RegExpEmpty.cpp
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#include "RegExpEmpty.h"

namespace regexp {
  
RegExpEmpty::RegExpEmpty() {
  // so that default constructor is available
}
  
RegExpEmpty::RegExpEmpty(const RegExpEmpty&) {
  // so that copy constructor is available
}

RegExpEmpty::RegExpEmpty(RegExpEmpty&&) noexcept {
  this->attachRegExp(NULL);
}

RegExpEmpty& RegExpEmpty::operator =(const RegExpEmpty&) {
  //this is actually different than default implementation
  return *this;
}

RegExpEmpty& RegExpEmpty::operator =(RegExpEmpty&&) noexcept {
  //this is actually different than default implementation
  return *this;
}

RegExpElement* RegExpEmpty::clone() const {
	return new RegExpEmpty(*this);
}

RegExpElement* RegExpEmpty::plunder() && {
	return new RegExpEmpty(std::move(*this));
}

bool RegExpEmpty::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool RegExpEmpty::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool RegExpEmpty::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool RegExpEmpty::operator<(const RegExpEmpty&) const {
	return false;
}

bool RegExpEmpty::operator==(const RegExpEmpty&) const {
	return true;
}

void RegExpEmpty::operator>>(std::ostream& out) const {
	out << "(RegExpEmpty)";
}

bool RegExpEmpty::testSymbol( const alphabet::Symbol & ) const {
	return false;
}

bool RegExpEmpty::attachRegExp(const RegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	return true;
}

void RegExpEmpty::computeMinimalAlphabet( std::set<alphabet::Symbol>&) const {

}

bool RegExpEmpty::containsEmptyString() const {
	return false;
}

bool RegExpEmpty::isEmpty() const {
	return true;
}

} /* namespace regexp */
