/*
 * RegExpEmpty.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_EMPTY_H_
#define REG_EXP_EMPTY_H_

#include "RegExpElement.h"

namespace regexp {

/**
 * Represents empty regular expression in the regular expression.
 */
class RegExpEmpty: public std::element<RegExpEmpty, RegExpElement> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
	RegExpEmpty();
	RegExpEmpty(const RegExpEmpty& other);
	RegExpEmpty(RegExpEmpty&& other) noexcept;
	RegExpEmpty& operator =(const RegExpEmpty& other);
	RegExpEmpty& operator =(RegExpEmpty&& other) noexcept;
  
	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const RegExpEmpty&) const;
	virtual bool operator==(const RegExpEmpty&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* REG_EXP_EMPTY_H_ */
