/*
 * RegExpEpsilon.h
 *
 *  Created on: Jan 30, 2014
 *      Author: Jan Travnicek
 */

#ifndef REG_EXP_EPSILON_H_
#define REG_EXP_EPSILON_H_

#include "RegExpElement.h"

namespace regexp {

/**
 * Represents epsilon in the regular expression.
 */
class RegExpEpsilon: public std::element<RegExpEpsilon, RegExpElement> {
protected:
	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;
	
	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	
	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;

public:
  
	RegExpEpsilon();
	RegExpEpsilon(const RegExpEpsilon& other);
	RegExpEpsilon(RegExpEpsilon&& other) noexcept;
	RegExpEpsilon& operator=(const RegExpEpsilon& other);
	RegExpEpsilon& operator=(RegExpEpsilon&& other) noexcept;
	
	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const RegExpEpsilon&) const;
	virtual bool operator==(const RegExpEpsilon&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;

	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* REG_EXP_EPSILON_H_ */
