/*
 * RegExpFromXMLParser.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_FROM_XML_PARSER_H_
#define REG_EXP_FROM_XML_PARSER_H_

#include "../FromXMLParser.hpp"
#include "RegExp.h"
#include "RegExpFeatures.h"
#include "RegExpElements.h"
#include "../sax/Token.h"
#include "../alphabet/Symbol.h"

namespace automaton {

class AutomatonFromXMLParser;

} /* namespace automaton */

namespace regexp {

/**
 * Parser used to get RegExp from XML parsed into list of tokens.
 */
class RegExpFromXMLParser : public alib::FromXMLParser<RegExp, FEATURES> {
	void parseAlphabet(std::list<sax::Token>& input, RegExp& regexp) const;

	RegExpElement* parseElement(std::list<sax::Token>& input) const;

	RegExpEpsilon* parseEpsilon(std::list<sax::Token>& input) const;
	RegExpEmpty* parseEmpty(std::list<sax::Token>& input) const;
	Iteration* parseIteration(std::list<sax::Token> &input) const;
	Alternation* parseAlternation(std::list<sax::Token> &input) const;
	Concatenation* parseConcatenation(std::list<sax::Token> &input) const;

	virtual RegExp parse(std::list<sax::Token>& input) const;
	virtual RegExp parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const;
public:
	virtual bool first(std::list<sax::Token>& input) const;

	friend class automaton::AutomatonFromXMLParser;
};

} /* namespace regexp */

#endif /* REG_EXP_FROM_XML_PARSER_H_ */
