/*
 * RegExpSymbol.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpSymbol.h"

namespace regexp {
  
RegExpSymbol::RegExpSymbol(const RegExpSymbol& other) : symbol(other.symbol) {
  
}

RegExpSymbol::RegExpSymbol(RegExpSymbol&& other) noexcept : symbol(std::move(other.symbol)) {
	this->attachRegExp(NULL);
}

RegExpSymbol& RegExpSymbol::operator=(const RegExpSymbol& other) {
  	if (this == &other) {
		return *this;
	}

	*this = RegExpSymbol(other);

	return *this;
}

RegExpSymbol& RegExpSymbol::operator=(RegExpSymbol&& other) {
	std::swap(this->symbol, other.symbol);
	std::swap(this->parentRegExp, other.parentRegExp);
	
	this->attachRegExp(other.parentRegExp);
	
	return *this;
}

RegExpSymbol::RegExpSymbol(const alphabet::Symbol& symbol) :
		symbol(symbol) {
}

RegExpSymbol::RegExpSymbol(alphabet::Symbol&& symbol) :
		symbol(std::move(symbol)) {
}

RegExpElement* RegExpSymbol::clone() const {
	return new RegExpSymbol(*this);
}

RegExpElement* RegExpSymbol::plunder() && {
	return new RegExpSymbol(std::move(*this));
}

bool RegExpSymbol::operator==(const alphabet::Symbol& other) const {
	return this->symbol == other;
}

bool operator==(const alphabet::Symbol& first, const RegExpSymbol& second) {
	return first == second.symbol;
}

bool RegExpSymbol::operator<(const RegExpElement& other) const {
	return other > *this;
}

bool RegExpSymbol::operator==(const RegExpElement& other) const {
	return other == *this;
}

bool RegExpSymbol::operator>(const RegExpElement& other) const {
	return other < *this;
}


bool RegExpSymbol::operator<(const RegExpSymbol& other) const {
	return symbol < other.symbol;
}

bool RegExpSymbol::operator==(const RegExpSymbol& other) const {
	return symbol == other.symbol;
}

void RegExpSymbol::operator>>(std::ostream& out) const {
	out << "(RegExpSymbol " << symbol << ")";
}

bool RegExpSymbol::containsEmptyString() const {
	return false;
}

bool RegExpSymbol::isEmpty() const {
	return false;
}

bool RegExpSymbol::testSymbol( const alphabet::Symbol & symbol ) const {
	return symbol == this->symbol;
}

bool RegExpSymbol::attachRegExp(const RegExp * regexp ) {
	if(this->parentRegExp == regexp) return true;
	this->parentRegExp = regexp;
	if(regexp == NULL) return true;
	return this->parentRegExp->getAlphabet().find(this->symbol) != this->parentRegExp->getAlphabet().end();
}

void RegExpSymbol::computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const {
	alphabet.insert(this->symbol);
}

const alphabet::Symbol& RegExpSymbol::getSymbol() const {
	return this->symbol;
}

} /* namespace regexp */

