/*
 * RegExpSymbol.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#ifndef REG_EXP_SYMBOL_H_
#define REG_EXP_SYMBOL_H_

#include "../label/Label.h"
#include "RegExpElement.h"
#include "../alphabet/LabeledSymbol.h"

namespace regexp {

/**
 * Represents symbol in the regular expression. Contains name of the symbol.
 */
class RegExpSymbol : public std::element<RegExpSymbol, RegExpElement> {
protected:
	alphabet::Symbol symbol;

	/**
	 * @copydoc RegExpElement::clone() const
	 */
	virtual RegExpElement* clone() const;
	
	virtual RegExpElement* plunder() &&;

	/**
	 * @copydoc RegExpElement::testSymbol() const
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
		
	/**
	 * @copydoc RegExpElement::attachRegExp()
	 */
	virtual bool attachRegExp ( const RegExp * regexp );
	
	/**
	 * @copydoc RegExpElement::computeMinimalAlphabet()
	 */
	virtual void computeMinimalAlphabet( std::set<alphabet::Symbol>& alphabet ) const;
public:
	explicit RegExpSymbol(const alphabet::Symbol& symbol);
	explicit RegExpSymbol(alphabet::Symbol&& symbol);
	
	RegExpSymbol(const RegExpSymbol& other);
	RegExpSymbol(RegExpSymbol&& other) noexcept;
	RegExpSymbol& operator=(const RegExpSymbol& other);
	RegExpSymbol& operator=(RegExpSymbol&& other);
	
	bool operator==(const alphabet::Symbol&) const;
	friend bool operator==(const alphabet::Symbol&, const RegExpSymbol&);

	virtual bool operator<(const RegExpElement&) const;
	virtual bool operator==(const RegExpElement&) const;
	virtual bool operator>(const RegExpElement&) const;

	virtual bool operator<(const RegExpSymbol&) const;
	virtual bool operator==(const RegExpSymbol&) const;
	
	/**
	 * @copydoc RegExpElement::operator>>() const
	 */
	virtual void operator>>(std::ostream& out) const;
	
	/**
	 * @copydoc RegExpElement::containsEmptyString() const
	 */
	virtual bool containsEmptyString() const;

	/**
	 * Returns the string representation of RegExp Symbol.
	 */
	const alphabet::Symbol& getSymbol() const;

	/**
	 * @copydoc RegExpElement::isEmpty() const
	 */
	virtual bool isEmpty() const;
};

} /* namespace regexp */

#endif /* REG_EXP_SYMBOL_H_ */

