/*
 * RegExpToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include <sstream>
#include <algorithm>
#include "RegExpToStringComposer.h"
#include "../alphabet/SymbolToStringComposer.h"

namespace regexp {

void RegExpToStringComposer::Visit(void* userData, const RegExp& regexp) {
	regexp.getRegExp().Accept(userData, *this);
}

void RegExpToStringComposer::Visit(void* userData, const RegExpElement::element_type& element) {
	element.Accept(userData, *this);
}

void RegExpToStringComposer::Visit(void* userData, const Alternation& alternation) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);

	Priority outerPriorityMinimum = out.first;
	if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) out.second << '(';
	bool first = true;
	for (const auto& element : alternation.getElements()) {
		if(first) {
			first = false;
		} else {
			out.second << '+';
		}
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		out.first = Priority::ALTERNATION;
		object.Accept(userData, *this);
	}
	if(outerPriorityMinimum == Priority::CONCATENATION || outerPriorityMinimum == Priority::FACTOR) out.second << ')';
}

void RegExpToStringComposer::Visit(void* userData, const Concatenation& concatenation) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);

	Priority outerPriorityMinimum = out.first;
	if(outerPriorityMinimum == Priority::FACTOR) out.second << '(';
	bool first = true;
	for (auto element : concatenation.getElements()) {
		if(first) 
			first = false;
		else
			out.second << ' ';
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		out.first = Priority::CONCATENATION;
		object.Accept(userData, *this);
	}
	if(outerPriorityMinimum == Priority::FACTOR) out.second << ')';
}

void RegExpToStringComposer::Visit(void* userData, const Iteration& iteration) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);

	const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(iteration.getElement());
	out.first = Priority::FACTOR;
	object.Accept(userData, *this);
	out.second << "*";
}

void RegExpToStringComposer::Visit(void* userData, const RegExpSymbol& symbol) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);
	
	alphabet::SymbolToStringComposer composer;
	out.second << composer.compose(symbol.getSymbol());
}

void RegExpToStringComposer::Visit(void* userData, const RegExpEpsilon&) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);
	out.second << "#E";
}

void RegExpToStringComposer::Visit(void* userData, const RegExpEmpty&) {
	std::pair<Priority, std::stringstream> &out = *((std::pair<Priority, std::stringstream>*) userData);
	out.second << "#0";
}

std::string RegExpToStringComposer::compose(const RegExp& regexp) {
	std::pair<Priority, std::stringstream> out;
	out.first = Priority::ALTERNATION;
	regexp.getRegExp().Accept((void*) &out, *this);
	return std::move(out.second).str();
}

} /* namespace regexp */
