/*
 * RegExpToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Martin Zak
 */

#include "RegExpToXMLComposer.h"

#include "../ToXMLComposers.h"

namespace regexp {

void RegExpToXMLComposer::Visit(void* userData, const RegExp& regexp) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("regexp", sax::Token::TokenType::START_ELEMENT));
	{
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
		for (const auto& symbol: regexp.getAlphabet()) {
			out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
		}
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	}
	regexp.getRegExp().Accept(userData, *this);
	out.push_back(sax::Token("regexp", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const RegExpElement::element_type& element) const {
	element.Accept(userData, *this);
}

void RegExpToXMLComposer::Visit(void* userData, const Alternation& alternation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("alternation", sax::Token::TokenType::START_ELEMENT));
	for (const auto& element : alternation.getElements()) {
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		object.Accept(userData, *this);
	}
	out.push_back(sax::Token("alternation", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const Concatenation& concatenation) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("concatenation", sax::Token::TokenType::START_ELEMENT));
	for (auto element : concatenation.getElements()) {
		const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(*element);
		object.Accept(userData, *this);
	}
	out.push_back(sax::Token("concatenation", sax::Token::TokenType::END_ELEMENT));

}

void RegExpToXMLComposer::Visit(void* userData, const Iteration& iteration) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("iteration", sax::Token::TokenType::START_ELEMENT));
	const RegExpElement::element_type& object = static_cast<const RegExpElement::element_type&>(iteration.getElement());
	object.Accept(userData, *this);
	out.push_back(sax::Token("iteration", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const RegExpSymbol& symbol) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol.getSymbol()));
}

void RegExpToXMLComposer::Visit(void* userData, const RegExpEpsilon&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("epsilon", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("epsilon", sax::Token::TokenType::END_ELEMENT));
}

void RegExpToXMLComposer::Visit(void* userData, const RegExpEmpty&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("empty", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("empty", sax::Token::TokenType::END_ELEMENT));
}

std::list<sax::Token> RegExpToXMLComposer::compose(const RegExp& regexp) const {
	std::list<sax::Token> out;
	Visit((void*) &out, regexp);
	return out;
}

} /* namespace regexp */
