/*
 * Visitor.h
 *
 *  Created on: Apr 05, 2014
 *      Author: Andrew Durward
 *      Modified: Jan Travnicek
 */

#ifndef VISITOR_H_
#define VISITOR_H_

namespace std {

// Visitor template declaration
template<typename... Types>
class visitor;

// specialization for single type
template<typename T>
class visitor<T> {
public:
    virtual void Visit(void* userData, const T & visitable) = 0;
};

// specialization for multiple types
template<typename T, typename... Types>
class visitor<T, Types...> : public visitor<Types...> {
public:
    // promote the function(s) from the base class
    using visitor<Types...>::Visit;

    virtual void Visit(void* userData, const T & visitable) = 0;
};

// Visitor template declaration
template<typename... Types>
class const_visitor;

// specialization for single type
template<typename T>
class const_visitor<T> {
public:
    virtual void Visit(void* userData, const T & visitable) const = 0;
};

// specialization for multiple types
template<typename T, typename... Types>
class const_visitor<T, Types...> : public const_visitor<Types...> {
public:
    // promote the function(s) from the base class
    using const_visitor<Types...>::Visit;

    virtual void Visit(void* userData, const T & visitable) const = 0;
};

template<typename... Types>
class elementBase {
public:
    typedef visitor<Types...> visitor_type;
    typedef const_visitor<Types...> const_visitor_type;
    typedef elementBase element_type;

    virtual void Accept(void* userData, visitor<Types...>& visitor) const = 0;
    virtual void Accept(void* userData, const const_visitor<Types...>& visitor) const = 0;
};

template<typename Derived, typename Base>
class element : public Base {
public:
    virtual void Accept(void* userData, typename Base::visitor_type& visitor) const {
        visitor.Visit(userData, static_cast<const Derived&>(*this));
    }
    virtual void Accept(void* userData, const typename Base::const_visitor_type& visitor) const {
        visitor.Visit(userData, static_cast<const Derived&>(*this));
    }
};

} /* namespace std */

#endif /* VISITOR_H_ */
