/*
 * CyclicString.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "CyclicString.h"
#include <cassert>
#include <sstream>
#include <algorithm>

#include "LinearString.h"
#include "Epsilon.h"

namespace string {

CyclicString::CyclicString(const std::vector<alphabet::Symbol>& data) : m_Data(data) {

}

CyclicString::CyclicString(std::vector<alphabet::Symbol>&& data) : m_Data(std::move(data)) {

}

bool CyclicString::testSymbol( const alphabet::Symbol & symbol ) const {
	return std::any_of(m_Data.begin(), m_Data.end(), [&](const alphabet::Symbol & s) { return s == symbol; } );
}

void CyclicString::computeMinimalAlphabet(std::set<alphabet::Symbol>& alphabet) const {
	alphabet.insert(m_Data.begin(), m_Data.end());
}

StringBase* CyclicString::clone() const {
	return new CyclicString(*this);
}

StringBase* CyclicString::plunder() && {
	return new CyclicString(std::move(*this));
}

bool CyclicString::operator <(const StringBase& other) const {
	return other > *this;
}

bool CyclicString::operator ==(const StringBase& other) const {
	return other == *this;
}

bool CyclicString::operator >(const StringBase& other) const {
	return other < *this;
}

const std::vector<alphabet::Symbol>& CyclicString::getContent() const {
	return this->m_Data;
}

bool CyclicString::isEmpty() const {
	return this->m_Data.size() == 0;
}

bool CyclicString::operator<(const LinearString& other) const {
	if(this->isEmpty() && other.isEmpty()) return false;
	return typeid(*this).before(typeid(other));
}

bool CyclicString::operator<(const CyclicString& other) const {
	return m_Data < other.m_Data; //TODO pouzit lexicograficky nejmensi rotaci
}

bool CyclicString::operator<(const Epsilon& other) const {
	if(this->isEmpty()) return false;
	return typeid(*this).before(typeid(other));
}

bool CyclicString::operator==(const LinearString& other) const {
	if(this->isEmpty() && other.isEmpty()) return true;
	return false;
}

bool CyclicString::operator==(const CyclicString& other) const {
	return m_Data == other.m_Data; //TODO pouzit lexikograficky nejmensi rotaci
}

bool CyclicString::operator==(const Epsilon&) const {
	if(this->isEmpty()) return true;
	return false;
}

void CyclicString::operator >>(std::ostream& out) const {
	if( this->isEmpty() ) {
		out << "(Epsilon)";
	} else {
		out << "(CyclicString ";
		for(const alphabet::Symbol& symbol : this->m_Data)
			out << symbol;
		out << ")";
	}
}

CyclicString::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str(); 
}

} /* namespace string */
