/*
 * Epsilon.h
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#ifndef EPSILON_H_
#define EPSILON_H_

#include <iostream>
#include <set>
#include <vector>

#include "../std/visitor.hpp"
#include "../alphabet/Symbol.h"
#include "StringBase.h"

namespace string {

/**
 * Represents epsilon. Regular expression is stored
 * as a tree of EpsilonElement.
 */
class Epsilon : public std::element<Epsilon, StringBase> {
protected:

	virtual bool testSymbol( const alphabet::Symbol & symbol ) const;
	virtual void computeMinimalAlphabet(std::set<alphabet::Symbol>&) const;
public:
	Epsilon();

	virtual StringBase* clone() const;
	virtual StringBase* plunder() &&;

	virtual bool operator <(const StringBase& other) const;
	virtual bool operator ==(const StringBase& other) const;
	virtual bool operator >(const StringBase& other) const;

	/**
	 * @return List of symbols forming string (const version).
	 */
	std::vector<alphabet::Symbol> getContent() const;

	/**
	 * @return true
	 */
	bool isEmpty() const;

	virtual bool operator<(const LinearString& other) const;
	virtual bool operator<(const CyclicString& other) const;
	virtual bool operator<(const Epsilon& other) const;

	virtual bool operator==(const LinearString& other) const;
	virtual bool operator==(const CyclicString& other) const;
	virtual bool operator==(const Epsilon& other) const;

	virtual void operator >>(std::ostream& out) const;

	virtual operator std::string() const;

	static Epsilon EPSILON;

};

} /* namespace string */

#endif /* EPSILON_H_ */

