/*
 * LinearString.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "LinearString.h"
#include <cassert>
#include <sstream>
#include <algorithm>

#include "CyclicString.h"
#include "Epsilon.h"

namespace string {

LinearString::LinearString(const std::vector<alphabet::Symbol>& data) : m_Data(data) {

}

LinearString::LinearString(std::vector<alphabet::Symbol>&& data) : m_Data(std::move(data)) {

}

bool LinearString::testSymbol( const alphabet::Symbol & symbol ) const {
	return std::any_of(m_Data.begin(), m_Data.end(), [&](const alphabet::Symbol & s) { return s == symbol; } );
}

void LinearString::computeMinimalAlphabet(std::set<alphabet::Symbol>& alphabet) const {
	alphabet.insert(m_Data.begin(), m_Data.end());
}

StringBase* LinearString::clone() const {
	return new LinearString(*this);
}

StringBase* LinearString::plunder() && {
	return new LinearString(std::move(*this));
}

bool LinearString::operator<(const StringBase& other) const {
	return other > *this;
}

bool LinearString::operator ==(const StringBase& other) const {
	return other == *this;
}

bool LinearString::operator >(const StringBase& other) const {
	return other < *this;
}

const std::vector<alphabet::Symbol>& LinearString::getContent() const {
	return this->m_Data;
}

bool LinearString::isEmpty() const {
	return this->m_Data.size() == 0;
}

bool LinearString::operator<(const LinearString& other) const {
	return m_Data < other.m_Data;
}

bool LinearString::operator<(const CyclicString& other) const {
	if(this->isEmpty() && other.isEmpty()) return false;
	return typeid(*this).before(typeid(other));
}

bool LinearString::operator<(const Epsilon& other) const {
	if(this->isEmpty()) return false;
	return typeid(*this).before(typeid(other));
}

bool LinearString::operator==(const LinearString& other) const {
	return m_Data == other.m_Data;
}

bool LinearString::operator==(const CyclicString& other) const {
	if(this->isEmpty() && other.isEmpty()) return true;
	return false;
}

bool LinearString::operator==(const Epsilon&) const {
	if(this->isEmpty()) return true;
	return false;
}

void LinearString::operator >>(std::ostream& out) const {
	if( this->isEmpty() ) {
		out << "(Epsilon)";
	} else {
		out << "(LinearString ";
		for(const alphabet::Symbol& symbol : this->m_Data)
			out << symbol;
		out << ")";
	}
}

LinearString::operator std::string () const {
	std::stringstream ss;
	ss << *this;
	return ss.str(); 
}

} /* namespace string */

