/*
 * String.h
 *
 *  Created on: Apr 10, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_H_
#define STRING_H_

#include "../std/visitor.hpp"
#include "StringBase.h"
#include <set>
#include "../alphabet/Symbol.h"

namespace string {

class StringBase;

/**
 * Wrapper around strings.
 */
class String {
protected:
	StringBase* string;
	std::set<alphabet::Symbol> alphabet;
public:
	explicit String();
	explicit String(const std::set<alphabet::Symbol>& alphabet, const StringBase& string);
	explicit String(const StringBase& string);
	explicit String(std::set<alphabet::Symbol>&& alphabet, StringBase&& string);
	explicit String(StringBase&& string);
	String(const String& other);
	String(String&&) noexcept;
	String& operator=(const String& other);
	String& operator=(String&& other) noexcept;
	virtual ~String() noexcept;

	const StringBase& getString() const;
	StringBase& getString();

	void setString(const StringBase& string);
	void setString(StringBase&& string);

	const std::set<alphabet::Symbol>& getAlphabet() const;
	bool addSymbolToAlphabet(const alphabet::Symbol & symbol);
	void setAlphabet(const std::set<alphabet::Symbol>& alphabet);
	bool removeSymbolFromAlphabet(const alphabet::Symbol & symbol);
	
	bool operator<(const String& other) const;

	bool operator!=(const String& other) const;

	bool operator==(const String& other) const;

	friend std::ostream& operator<<(std::ostream& os, const String& string);

	operator std::string () const;
};

} /* namespace string */

#endif /* STRING_H_ */

