/*
 * StringBase.h
 *
 *  Created on: Mar 26, 2013
 *      Author: Jan Travnicek
 */

#ifndef STRING_BASE_H_
#define STRING_BASE_H_

#include "../label/LabelBase.h"
#include <ostream>
#include <set>
#include "../alphabet/Symbol.h"
#include "String.h"
	
namespace string {

class String;

class LinearString;
class CyclicString;
class Epsilon;

/**
 * Represents string in an alphabet.
 */
class StringBase : public std::elementBase<Epsilon, LinearString, CyclicString> {
protected:
	const String* parentString;

	/**
	 * Traverses the string tree looking wheter a particular Symbol is used in the string.
	 *
	 * @param symbol to test wheter used in string
	 * @return true if symbol is used by the element and its successor
	 */
	virtual bool testSymbol( const alphabet::Symbol & symbol ) const = 0;

	/**
	 * Attaches the string to this instance
	 * @param string parent string to attach as parent
	 * @return true if symbols used in string are in the regexp's alphabet
	 */
	virtual bool attachString ( const String * string );

	/**
	 * Traverses the string tree computing minimal alphabet needed by the string
	 *
	 * @param alphabet All alphabet symbols encountered are added into this set
	 * @return true if symbol is used by the element and its successor
	 */
	virtual void computeMinimalAlphabet(std::set<alphabet::Symbol>&) const = 0;
public:
	StringBase();
	virtual ~StringBase() noexcept;

	virtual StringBase* clone() const = 0;
	virtual StringBase* plunder() && = 0; 

	virtual bool operator <(const StringBase& other) const = 0;
	virtual bool operator ==(const StringBase& other) const = 0;
	virtual bool operator >(const StringBase& other) const = 0;
	bool operator !=(const StringBase& other) const;

	virtual bool operator==(const LinearString& other) const = 0;
	virtual bool operator==(const CyclicString& other) const = 0;
	virtual bool operator==(const Epsilon& other) const = 0;

	virtual bool operator<(const LinearString& other) const = 0;
	virtual bool operator<(const CyclicString& other) const = 0;
	virtual bool operator<(const Epsilon& other) const = 0;


	/**
	 * Prints XML representation of the String to the output stream.
	 * @param out output stream to which print the String
	 * @param string String to print
	 */
	friend std::ostream& operator<<(std::ostream&, const StringBase&);

	virtual void operator>>(std::ostream& out) const = 0;

	virtual operator std::string () const = 0;

	friend class String;

	friend class Epsilon;
	friend class LinearString;
	friend class CyclicString;
};

} /* namespace string */

#endif /* STRING_BASE_H_ */

