/*
 * StringFromXMLParser.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringFromXMLParser.h"
#include "../sax/ParserException.h"
#include "Epsilon.h"
#include "LinearString.h"
#include "CyclicString.h"

#include "../FromXMLParsers.h"

namespace string {

String StringFromXMLParser::parse(std::list<sax::Token>& input) const {
	return parse(input, std::set<FEATURES>({FEATURES::LINEAR, FEATURES::CYCLIC, FEATURES::EPSILON}));
}

String StringFromXMLParser::parse(std::list<sax::Token>& input, const std::set<FEATURES>& features) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "string");
	
	String string;
	parseAlphabet(input, string);
	parseContent(input, string, features);
	
	popToken(input, sax::Token::TokenType::END_ELEMENT, "string");

	return string;
}

void StringFromXMLParser::parseContent(std::list<sax::Token>& input, String& string, const std::set<FEATURES>& features) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "Epsilon")) {
		if(!features.count(FEATURES::EPSILON)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "Epsilon");
		popToken(input, sax::Token::TokenType::END_ELEMENT, "Epsilon");
		string.setString(Epsilon());
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "LinearString")) {
		if(!features.count(FEATURES::LINEAR)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "LinearString");
		std::vector<alphabet::Symbol> data = parseContentData(input);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "LinearString");
		string.setString(LinearString(data));
	} else if(isToken(input, sax::Token::TokenType::START_ELEMENT, "CyclicString")) {
		if(!features.count(FEATURES::CYCLIC)) throw exception::AlibException();
		popToken(input, sax::Token::TokenType::START_ELEMENT, "CyclicString");
		std::vector<alphabet::Symbol> data = parseContentData(input);
		popToken(input, sax::Token::TokenType::END_ELEMENT, "CyclicString");
		string.setString(CyclicString(data));
	} else {
		throw sax::ParserException(sax::Token("Epsilon, LinearString, CyclicString", sax::Token::TokenType::START_ELEMENT), input.front());
	}
}

void StringFromXMLParser::parseAlphabet(std::list<sax::Token> &input, String& string) const {
	popToken(input, sax::Token::TokenType::START_ELEMENT, "alphabet");
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		string.addSymbolToAlphabet(alib::FromXMLParsers::symbolParser.parse(input));
	}
	popToken(input, sax::Token::TokenType::END_ELEMENT, "alphabet");
}

bool StringFromXMLParser::first(std::list<sax::Token>& input) const {
	if(isToken(input, sax::Token::TokenType::START_ELEMENT, "string")) {
		return true;
	} else {
		return false;
	}
}

std::vector<alphabet::Symbol> StringFromXMLParser::parseContentData(std::list<sax::Token>& input) const {
	std::vector<alphabet::Symbol> data;
	while (isTokenType(input, sax::Token::TokenType::START_ELEMENT)) {
		data.push_back(alib::FromXMLParsers::symbolParser.parse(input));
	}
	return data;
}

} /* namespace string */
