/*
 * StringToStringComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringToStringComposer.h"
#include <algorithm>
#include "../alphabet/Symbol.h"
#include "../alphabet/SymbolToStringComposer.h"
#include "CyclicString.h"
#include "LinearString.h"

namespace string {

void StringToStringComposer::Visit(void* userData, const CyclicString& string) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << "<";
	for(const auto& symbol : string.getContent()) {
		alphabet::SymbolToStringComposer composer;
		out << composer.compose(symbol);
	}
	out << ">";
}

void StringToStringComposer::Visit(void* userData, const LinearString& string) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << "\"";
	for(const auto& symbol : string.getContent()) {
		alphabet::SymbolToStringComposer composer;
		out << composer.compose(symbol);
	}
	out << "\"";
}

void StringToStringComposer::Visit(void* userData, const Epsilon&) {
	std::stringstream &out = *((std::stringstream*) userData);

	out << "#E";
}

void StringToStringComposer::Visit(void* userData, const String& string) {
	string.getString().Accept(userData, *this);

}

std::string StringToStringComposer::compose(const String& string) {
	std::stringstream out;
	Visit((void*) &out, string);
	return std::move(out).str();
}

} /* namespace string */
