/*
 * StringToXMLComposer.cpp
 *
 *  Created on: Nov 23, 2013
 *      Author: Jan Travnicek
 */

#include "StringToXMLComposer.h"
#include "CyclicString.h"
#include "LinearString.h"
#include "Epsilon.h"

#include "../ToXMLComposers.h"

namespace string {

void StringToXMLComposer::Visit(void* userData, const Epsilon&) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("Epsilon", sax::Token::TokenType::START_ELEMENT));
	out.push_back(sax::Token("Epsilon", sax::Token::TokenType::END_ELEMENT));
}

void StringToXMLComposer::Visit(void* userData, const CyclicString& string) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("CyclicString", sax::Token::TokenType::START_ELEMENT));
	for(const auto& symbol : string.getContent()) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("CyclicString", sax::Token::TokenType::END_ELEMENT));
}

void StringToXMLComposer::Visit(void* userData, const LinearString& string) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);

	out.push_back(sax::Token("LinearString", sax::Token::TokenType::START_ELEMENT));
	for(const auto& symbol : string.getContent()) {
		out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
	}
	out.push_back(sax::Token("LinearString", sax::Token::TokenType::END_ELEMENT));
}

void StringToXMLComposer::Visit(void* userData, const String& string) const {
	std::list<sax::Token> &out = *((std::list<sax::Token>*) userData);
	
	out.push_back(sax::Token("string", sax::Token::TokenType::START_ELEMENT));
	{
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::START_ELEMENT));
		for (const auto& symbol: string.getAlphabet()) {
			out.splice(out.end(), alib::ToXMLComposers::symbolComposer.compose(symbol));
		}
		out.push_back(sax::Token("alphabet", sax::Token::TokenType::END_ELEMENT));
	}
	string.getString().Accept(userData, *this);
	out.push_back(sax::Token("string", sax::Token::TokenType::END_ELEMENT));
}

std::list<sax::Token> StringToXMLComposer::compose(const String& string) const {
	std::list<sax::Token> out;
	Visit((void*) &out, string);
	return out;
}

} /* namespace string */
