#include <list>
#include "StringTest.h"

#include "sax/SaxParseInterface.h"
#include "sax/SaxComposeInterface.h"

#include "string/String.h"
#include "string/LinearString.h"
#include "string/CyclicString.h"
#include "string/Epsilon.h"
#include "string/StringFromStringParser.h"
#include "string/StringToStringComposer.h"
#include "string/StringFromXMLParser.h"
#include "string/StringToXMLComposer.h"

#include "factory/StringFactory.h"

#include "alphabet/Symbol.h"
#include "alphabet/LabeledSymbol.h"

#include "label/StringLabel.h"
#include "label/Label.h"

#define CPPUNIT_IMPLY(x, y) CPPUNIT_ASSERT(!(x) || (y))

CPPUNIT_TEST_SUITE_REGISTRATION( StringTest );

void StringTest::setUp() {
}

void StringTest::tearDown() {
}

void StringTest::testCopyConstruct() {
  string::String string;
  string.setAlphabet({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("3"))))});
  string.setString(string::LinearString({
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))),
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))),
			  alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))))
			  }));
  
  string::String string2(string);
  
  CPPUNIT_ASSERT( string == string2 );
  
  string::String string3(std::move(string));
  
  CPPUNIT_ASSERT( string2 == string3 );
}

void StringTest::testEqual() {
  {
    std::string input = "\"aa\"";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
  {
    std::string input = "\"'aaaa''aaa'\"";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
  {
    std::string input = "#E";
    std::stringstream inputs(input);

    string::StringFromStringParser parser(inputs);
    string::String string = parser.parseValue();

    string::StringToStringComposer composer;
    std::string output = composer.compose(string);
    std::stringstream outputs(output);

    CPPUNIT_ASSERT( input == output );

    string::StringFromStringParser parser2(outputs);
    string::String string2 = parser2.parseValue();

    CPPUNIT_ASSERT( string == string2 );
  }
}

void StringTest::testXMLParser() {

  string::String string;
  string.setAlphabet({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))), alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("3"))))});
  string.setString(string::LinearString({alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1")))),
					alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("2")))),
					alphabet::Symbol(alphabet::LabeledSymbol(label::Label(label::StringLabel("1"))))}
		  )
		);

  {
	string::StringToXMLComposer composer;
	std::list<sax::Token> tokens = composer.compose(string);
	std::string tmp;
	sax::SaxComposeInterface::printMemory(tmp, tokens);
  
	std::list<sax::Token> tokens2;
	sax::SaxParseInterface::parseMemory(tmp, tokens2);
	string::StringFromXMLParser parser;
	string::String string2 = parser.parseValue(tokens2);
  
	CPPUNIT_ASSERT( string == string2 );
  }
  {
	std::string tmp = string::StringFactory::toString(string);
	string::String string2 = string::StringFactory::fromString(tmp);
  
	CPPUNIT_ASSERT( string == string2 );
  }
}

void StringTest::testCompare() {
  string::String linear({}, string::LinearString({}));
  string::String cyclic({}, string::CyclicString({}));
  string::String epsilon({}, string::Epsilon({}));

  CPPUNIT_ASSERT(linear == epsilon);
  CPPUNIT_ASSERT(cyclic == epsilon);
  CPPUNIT_ASSERT(cyclic == linear);
  CPPUNIT_ASSERT(epsilon == linear);
  CPPUNIT_ASSERT(epsilon == cyclic);
  CPPUNIT_ASSERT(linear == cyclic);
}

