#include <iostream>
#include <getopt.h>

#include <AutomatonFactory.h>
#include <GrammarFactory.h>
#include <automaton/AutomatonParser.h>
#include <grammar/GrammarParser.h>

#include <AlibException.h>

#include <sax/SaxInterface.h>
#include <sax/ParserException.h>

#include "grammar/ContextFreeGrammarTransformations.h"
#include "automaton/TrimFSM.h"

using namespace std;
using namespace automaton;
using namespace grammar;
using namespace alib;
using namespace sax;

void help( void )
{
    cout << "atrim 0.01" << endl;
    cout << "Removes unreachable and dead states from FSM, productive and unreachable nonterminals from CFG." << endl;
    cout << "Usage: atrim [-d] [-r]" << endl << endl;
    cout << "If neither --dead nor --unreachable option is used, both dead and unreachable states are removed." << endl;
    cout << endl;
    cout << " -d, --dead \t Removes dead states only (works with FSM only)." << endl;
    cout << " -u, --unreachable \t Removes unreachable states only. (works with FSM or CFG)." << endl;
    cout << " -h, --help \t shows this." << endl;

    cout << endl;
}

int main(int argc, char* argv[])
{
    bool del_dead_only = true, del_unreachable_only = true;

    static struct option long_options[] = {
      {"help",        no_argument, NULL,    'h'},
      {"dead",        no_argument, NULL,    'd'},
      {"unreachable", no_argument, NULL,    'u'},
      {0, 0, 0, 0}
    };

    int long_index = 0, opt = 0;

    while( ( opt = getopt_long( argc, argv, "hdu", long_options, & long_index ) ) != -1 )
    {
        switch( opt )
        {
        case 'd':
            del_dead_only = true;
            break;
        case 'u':
            del_unreachable_only = true;
        break;

        case 'v':
        case 'h':
        default:
            help( );
            return 0;
        }
    }

    list<Token> tokens;
    if(optind == argc)
    {
        string input(istreambuf_iterator<char>(cin), (istreambuf_iterator<char>()));
        SaxInterface::parseMemory(input, tokens);
    }
    else
    {
        SaxInterface::parseFile(argv[optind],tokens);
    }


    if (tokens.front().getData() == "automaton")
    {
        FSM fsm = AutomatonFactory::buildFSM( AutomatonParser::parse(tokens) );

        if( ! del_dead_only && ! del_unreachable_only )
            del_dead_only = del_unreachable_only = true;

        if( del_unreachable_only )
            fsm = TrimFSM::removeUnreachableStates( fsm );
        if( del_dead_only )
            fsm = TrimFSM::removeDeadStates( fsm );

        fsm.toXML( cout );
    }
    else if( tokens.front().getData() == "grammar" )
    {
        ContextFreeGrammar cfg = GrammarFactory::buildContextFreeGrammar( GrammarParser::parse( tokens ) );

        if( del_unreachable_only )
            cfg = ContextFreeGrammarTransformations::removeUnreachableSymbols( cfg );
        else
            cfg = ContextFreeGrammarTransformations::removeUnproductiveSymbols( cfg );

        cfg.toXML( cout );
    }
    else
    {
        throw AlibException( "Input not recognized." );
        return 1;
    }

    return 0;
}
