/*
 * ContextFreeGrammarTransformations.h
 *
 *  Created on: 22. 3. 2014
 *      Author: tomas
 */

#ifndef CONTEXTFREEGRAMMARTRANSFORMATIONS_H_
#define CONTEXTFREEGRAMMARTRANSFORMATIONS_H_

#include <algorithm>
#include <deque>
#include <set>

#include <AlibException.h>
#include <grammar/ContextFree/ContextFreeGrammar.h>

#define isInSet(x,set) ( (set).find((x)) != (set).end())

/**
 * Implements algorithms from Melichar, chapter 3.3
 */
class ContextFreeGrammarTransformations
{
public:
    /*
     * Melichar 3.6 - decides whether L( grammar ) = \0
     *
     * Severals steps implemented in method ContextFreeGrammarTransformations::getProductiveNonTerminals();
     * @see getProductiveNonTerminals
     */
    static bool isLanguageEmpty( const grammar::ContextFreeGrammar & grammar );

    /*
     * Removes unreachable symbols - Melichar 3.9
     */
    static grammar::ContextFreeGrammar removeUnreachableSymbols( const grammar::ContextFreeGrammar & grammar );

    /**
     * Removes unproductive (or useless - terminology) symbols - Melichar 3.12
     */
    static grammar::ContextFreeGrammar removeUnproductiveSymbols( const grammar::ContextFreeGrammar & grammar );

private:
    /**
     * Implements steps 1 through 3 in Melichar 3.6
     */
    static std::set<alphabet::Symbol> getProductiveNonTerminals( const grammar::ContextFreeGrammar & grammar );
};

#endif /* CONTEXTFREEGRAMMARTRANSFORMATIONS_H_ */
