/*
 * RegExpDerivation.h
 *
 *  Created on: 19. 1. 2014
 *      Author: tomas
 */

#ifndef REGEXPDERIVATION_H_
#define REGEXPDERIVATION_H_

#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

#include <AlibException.h>

#include <list>


/**
 * Calculates derivation of regular expression.
 *
 * Sources:
 *  - Melichar, definition 2.91 in chapter 2.4.3
 *  - Brzozowski, J. A. - Derivatives of regular expressions (1964)
 */
class RegExpDerivation
{
public:
    /**
     * @param re RegExp to derivate
     */
    RegExpDerivation( const regexp::RegExp & re );

    /**
     * returns derivation of regexp over word dString
     * @param dString list of RegExpElements representing word. Derivation will be over this word.
     * @return Derivation of regexp over dString
     */
    regexp::RegExp derivation( const std::list<alphabet::Symbol> & dString ) const;

    /**
     * returns derivation of regexp over one-symbol word
     * @param symbol one-symbol word
     * @return Derivation of regexp over word
     */
    regexp::RegExp derivation( const alphabet::Symbol & symbol ) const;

private:
    regexp::RegExpElement * derivation( const regexp::RegExpElement * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Alternation * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Concatenation * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::Iteration * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpSymbol * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpEpsilon * element, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * derivation( const regexp::RegExpEmpty * element, const alphabet::Symbol & dSymbol ) const;

    /**
     * stores original regexp
     */
    const regexp::RegExp m_re;
};

#endif /* REGEXPDERIVATION_H_ */
