/*
 * RegExpIntegral.cpp
 *
 *  Created on: 24. 2. 2014
 *      Author: tomas
 */

#include "RegExpIntegral.h"

using namespace alib;
using namespace regexp;

RegExpIntegral::RegExpIntegral( const RegExp & re ) : m_re( re )
{

}

RegExp RegExpIntegral::integral( const list<Symbol> & dString ) const
{
    // TODO: change dString to WORD object when implemented.

    const RegExpElement * oldRegExp = m_re.getRegExp( )->clone( ), * integralRegExp;

    for( auto it = dString.rbegin( ); it != dString.rend( ); it ++ )
    {
        const auto & dSymbol = * it;

        integralRegExp = integral( oldRegExp, dSymbol );

        delete oldRegExp;
        oldRegExp = integralRegExp;
    }

    RegExp ret ( oldRegExp );
    delete oldRegExp;
    return ret;
}

RegExp RegExpIntegral::integral ( const Symbol & symbol ) const
{
    RegExp ret = integral( m_re.getRegExp( ), symbol );
    return ret;
}

RegExpElement * RegExpIntegral::integral( const RegExpElement * node, const Symbol & dSymbol ) const
{
    const Alternation * alternation = dynamic_cast<const Alternation*>( node );
    const Concatenation * concatenation = dynamic_cast<const Concatenation*>( node );
    const Iteration * iteration = dynamic_cast<const Iteration*>( node );
    const RegExpSymbol * symbol = dynamic_cast<const RegExpSymbol*>( node );
    const RegExpEpsilon * eps = dynamic_cast<const RegExpEpsilon*>( node );
    const RegExpEmpty * empty = dynamic_cast<const RegExpEmpty*>( node );

    if( alternation )
        return integral( alternation, dSymbol );
    if( concatenation )
        return integral( concatenation, dSymbol);
    if( iteration )
        return integral( iteration, dSymbol );
    if( symbol )
        return integral( symbol, dSymbol );
    if( eps )
        return integral( eps, dSymbol );
    if( empty )
        return integral( empty, dSymbol );

    throw AlibException( "RegExpDerivation::derivation() - unknown RegExpElement node" );
}

RegExpElement * RegExpIntegral::integral( const Alternation * node, const Symbol & dSymbol ) const
{
    Alternation * alt = new Alternation( );

    for( const auto & child : node->getElements( ) )
        alt->getElements( ).push_back( integral( child, dSymbol ) );

    return alt;
}

RegExpElement * RegExpIntegral::integral( const Concatenation * node, const Symbol & dSymbol ) const
{
    Concatenation * concat = new Concatenation( );
    concat->getElements( ).push_back( new RegExpSymbol( dSymbol.getSymbol( ) ) );

    for( const auto & child : node->getElements( ) )
        concat->getElements( ).push_back( child->clone( ) );

    return concat;
}

RegExpElement * RegExpIntegral::integral( const Iteration * node, const Symbol & dSymbol ) const
{
    Concatenation * concat = new Concatenation( );
    concat->getElements( ).push_back( new RegExpSymbol( dSymbol.getSymbol( ) ) );
    concat->getElements( ).push_back( node->getElement( )->clone( ) );
    return concat;
}

RegExpElement * RegExpIntegral::integral( const RegExpSymbol * node, const Symbol & dSymbol ) const
{
    Concatenation * concat = new Concatenation( );
    concat->getElements( ).push_back( new RegExpSymbol( dSymbol.getSymbol( ) ) );
    concat->getElements( ).push_back( node->clone( ) );
    return concat;
}

RegExpElement * RegExpIntegral::integral( const RegExpEpsilon * node, const Symbol & dSymbol ) const
{
    return new RegExpSymbol( dSymbol.getSymbol( ) );
}

RegExpElement * RegExpIntegral::integral( const RegExpEmpty * node, const Symbol & dSymbol ) const
{
    return new RegExpEmpty( );
}
