/*
 * RegExpIntegral.h
 *
 *  Created on: 24. 2. 2014
 *      Author: tomas
 */

#ifndef REGEXPINTEGRAL_H_
#define REGEXPINTEGRAL_H_

#include <AlibException.h>

#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

/**
 * Computes integral of regular expression as defined in Melichar, 2.93 (chapter 2.4.4)
 */
class RegExpIntegral
{
public:
    /**
     * @param re RegExp to integrate
     */
    RegExpIntegral( const regexp::RegExp & re );

    /**
     * returns integral of regexp over word dString
     * @param dString list of RegExpElements representing word. Integral will be over this word.
     * @see RegExpElement
     * @return Integral of regexp over dString
     */
    regexp::RegExp integral( const std::list<alphabet::Symbol> & dString ) const;

    /**
     * returns integral of regexp over one-symbol word
     * @param symbol one-symbol word
     * @return integral of regexp over word
     */
    regexp::RegExp integral( const alphabet::Symbol & symbol ) const;

private:
    regexp::RegExpElement * integral( const regexp::RegExpElement * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::Alternation * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::Concatenation * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::Iteration * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::RegExpSymbol * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::RegExpEpsilon * node, const alphabet::Symbol & dSymbol ) const;
    regexp::RegExpElement * integral( const regexp::RegExpEmpty * node, const alphabet::Symbol & dSymbol ) const;

    /**
     * stores original regular expression
     */
    const regexp::RegExp m_re;
};

#endif /* REGEXPINTEGRAL_H_ */
